# -*- coding: utf-8 -*-
# Arkanae III
# Copyright (C) 2008-2009 Jean-Baptiste LAMY
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import random

from arkanae3.base import *


class MonsterController(object):
  vision_sq = 30.0
  def find_target(self):
    target = None
    best_dist = self.vision_sq
    
    for mobile in self.character.level.mobiles:
      if (not mobile is self.character) and isinstance(mobile, BaseCharacter) and (self.character.get_relation(mobile) == -1) and (not mobile.non_targetable) and (mobile.life > 0.0):
        if mobile.flag & FLAG_TRAITOR:
          dangle = abs(self.character.angle - vector2angle(mobile.i - self.character.i, mobile.j - self.character.j))
          if dangle > 180.0: dangle = 360.0 - dangle
          
          if (dangle >  80.0): continue # The traitor is in the monster's back
          
        di = (self.character.i - mobile.i)
        dj = (self.character.j - mobile.j)
        dist = di * di + dj * dj
        if dist < best_dist:
          target = mobile
          best_dist = dist

    return target
    
    
class EyeController(object):
  def find_target(self):
    targets = []
    for mobile in self.character.level.mobiles:
      if isinstance(mobile, BaseHero) and (not mobile.non_targetable) and (mobile.life > 0.0):
        dangle = abs(self.character.angle - vector2angle(mobile.i - self.character.i, mobile.j - self.character.j))
        if dangle > 180.0: dangle = 360.0 - dangle
        if (dangle <  80.0) and (mobile.get_model_name().split("_")[-1] != self.character.get_model_name().split("_")[-1]):
          targets.append(mobile)
          
    if targets: return random.choice(targets)
    
    
class EarController(object):
  ear_radius_sq = 5.0
  def find_target(self):
    targets = []
    for mobile in self.character.level.mobiles:
      if isinstance(mobile, BaseHero) and (not mobile.non_targetable) and (mobile.life > 0.0):
        if ((mobile.i - self.character.i) ** 2 + (mobile.j - self.character.j) ** 2 < self.ear_radius_sq) and (mobile.get_model_name().split("_")[-1] != self.character.get_model_name().split("_")[-1]):
          targets.append(mobile)
          
    if targets: return random.choice(targets)
    
    

class AIController(BaseController):
  def __init__(self):
    self.target     = None
    self.generators = [self]
    
  def generate_actions(self):
    if self.target and self.target.discussion:
      self.character.send_action(ACTION_STOP_MOVING)
      return
    
    try:
      self.generators[-1].next()
    except StopIteration:
      del self.generators[-1]
      return self.generate_actions()
    
  def next(self):
    if not self.target: self.target = self.find_target()
    
    if self.target:
      if (not self.target.level is self.character.level) or (self.target.life <= 0.0): self.target = None
      else:
        self.generators.append(self.next_with_target())
    else:
      self.generators.append(self.next_with_no_target())
      
      
class ChevalierEchassienController(MonsterController, AIController):
  def next_with_no_target(self):
    if not self.character.faction == 0:
      self.character.send_action(ACTION_TURN_LEFT)
      yield
      yield
      yield
      yield
      self.character.send_action(ACTION_STOP_MOVING)
    for i in range(25): yield
    
  def next_with_target(self):
    yield
    self.character.send_action(ACTION_GOTO_STRIKE + struct.pack("!fff", self.target.i, self.target.j, vector2angle(self.target.i - self.character.i, self.target.j - self.character.j)) + self.target.dumpsuid())
    yield
    while 1:
      if   self.character.current_action == ACTION_STRIKE:
        if self.character.animation_pos == 1:
          self.character.send_action(ACTION_STOP_MOVING)
          
      elif self.character.current_action == ACTION_STOP_MOVING:
      #  for i in range(20): yield
        break
      yield


class TodoController(MonsterController, AIController): # XXX
  def next_with_no_target(self):
    if not self.character.faction == 0:
      self.character.send_action(ACTION_TURN_LEFT)
      yield
      yield
      yield
      yield
      self.character.send_action(ACTION_STOP_MOVING)
    for i in range(25): yield
    
  def next_with_target(self):
    self.character.send_action(ACTION_GOTO_STRIKE + struct.pack("!fff", self.target.i, self.target.j, vector2angle(self.target.i - self.character.i, self.target.j - self.character.j)) + self.target.dumpsuid())
    yield
    while 1:
      if   self.character.current_action == ACTION_STRIKE:
        if self.character.animation_pos == 1:
          self.character.send_action(ACTION_STOP_MOVING)
          
      elif self.character.current_action == ACTION_STOP_MOVING:
        self.character.send_action(ACTION_STRIKE_ONE)
        for i in range(20): yield
        if self.character.life > 0.0:
          self.character.set_current_state_importance(2) # Angle is changed
          angle = self.character.angle = (self.character.angle + 180.0) % 360.0
          ri, rj = angle2vector(angle, 2.0)
          self.character.send_action(ACTION_GOTO + struct.pack("!fff", self.character.i + ri, self.character.j + rj, angle) + self.target.dumpsuid())
          while self.character.current_action != ACTION_STOP_MOVING: yield
        break
      yield
      
class InvocateurLigeController(TodoController): pass
class SorcierFideController(TodoController): pass
class PretreEchassienController(TodoController): pass
class VagabonController(TodoController): pass
class VoleurEchassienController(TodoController): pass
class BarbareFideController(TodoController): pass

class BarbareFide0Controller(MonsterController, AIController):
  vision_sq = 28.0
  def next_with_no_target(self):
    if not self.character.faction == 0:
      self.character.send_action(ACTION_TURN_LEFT)
      yield
      yield
      yield
      yield
      self.character.send_action(ACTION_STOP_MOVING)
    for i in range(25): yield
    
  def next_with_target(self):
    yield
    self.character.send_action(ACTION_GOTO_STRIKE + struct.pack("!fff", self.target.i, self.target.j, vector2angle(self.target.i - self.character.i, self.target.j - self.character.j)) + self.target.dumpsuid())
    for i in range(10): yield
    
    while 1:
      if   self.character.current_action == ACTION_STRIKE:
        if self.character.animation_pos >= 16:
          self.character.send_action(ACTION_STOP_MOVING)
      else:
        break
      yield

class BarbareFide1Controller(MonsterController, AIController):
  vision_sq = 28.0
  def next_with_no_target(self):
    for i in range(25): yield
    if not self.character.faction == 0:
      self.character.send_action(ACTION_TURN_RIGHT)
      yield
      yield
      yield
      yield
      self.character.send_action(ACTION_STOP_MOVING)
      
  def next_with_target(self):
    yield
    self.character.send_action(ACTION_GOTO_STRIKE + struct.pack("!fff", self.target.i, self.target.j, vector2angle(self.target.i - self.character.i, self.target.j - self.character.j)) + self.target.dumpsuid())
    for i in range(12): yield
    
    while 1:
      if   self.character.current_action == ACTION_STRIKE:
        if self.character.animation_pos >= 16:
          self.character.send_action(ACTION_STOP_MOVING)
      else:
        break
      yield
      
    if self.character.life > 0.5 * self.character.max_life:
      for i in range(4): yield
      

      
     
      
class OrkController(MonsterController, AIController):
  vision_sq = 16.0
  def next_with_no_target(self):
    self.character.send_action(ACTION_STOP_MOVING)
    for i in range(30): yield
    
  def next_with_target(self):
    yield
    self.character.send_action(ACTION_GOTO_STRIKE + struct.pack("!fff", self.target.i, self.target.j, vector2angle(self.target.i - self.character.i, self.target.j - self.character.j)) + self.target.dumpsuid())
    yield
    while 1:
      if   self.character.current_action == ACTION_STRIKE:
        if self.character.animation_pos == 1:
          self.character.send_action(ACTION_STOP_MOVING)
          
      elif self.character.current_action == ACTION_STOP_MOVING:
        for i in range(24): yield
        break
      yield


class Ork2Controller(MonsterController, AIController):
  vision_sq = 16.0
  def next_with_no_target(self):
    self.character.send_action(ACTION_STOP_MOVING)
    for i in range(30): yield
    
  def next_with_target(self):
    yield
    self.character.send_action(ACTION_GOTO_STRIKE + struct.pack("!fff", self.target.i, self.target.j, vector2angle(self.target.i - self.character.i, self.target.j - self.character.j)) + self.target.dumpsuid())
    yield
    while 1:
      if   self.character.current_action == ACTION_STRIKE:
        if self.character.animation_pos == 1:
          self.character.send_action(ACTION_STOP_MOVING)
          
      elif self.character.current_action == ACTION_STOP_MOVING:
        for i in range(14): yield
        break
      yield


class Ork3Controller(MonsterController, AIController):
  vision_sq = 100.0
  def next_with_no_target(self):
    self.character.send_action(ACTION_STOP_MOVING)
    for i in range(30): yield
    
  def next_with_target(self):
    while self.character.k:
      self.character.send_action(ACTION_STOP_MOVING)
      yield
    yield
    self.character.send_action(ACTION_GOTO_STRIKE + struct.pack("!fff", self.target.i, self.target.j, vector2angle(self.target.i - self.character.i, self.target.j - self.character.j)) + self.target.dumpsuid())
    yield
    while 1:
      if   self.character.current_action == ACTION_STRIKE:
        if self.character.animation_pos == 1:
          self.character.send_action(ACTION_STOP_MOVING)
          
      elif self.character.current_action == ACTION_STOP_MOVING:
        for i in range(20): yield
        break
      yield


class FruitController(EarController, AIController):
  ear_radius_sq = 16.0
  def next_with_no_target(self):
    self.character.send_action(ACTION_STOP_MOVING)
    for i in range(30): yield
    
  def next_with_target(self):
    self.character.send_action(ACTION_GOTO_STRIKE + struct.pack("!fff", self.target.i, self.target.j, vector2angle(self.target.i - self.character.i, self.target.j - self.character.j)) + self.target.dumpsuid())
    
    for i in range(random.randint(3, 7)):
      yield

    while 1:
      if (self.character.i - self.target.i) ** 2 + (self.character.j - self.target.j) ** 2 <= 2.9:
        break
      yield
    self.character.send_action(ACTION_STRIKE_ONE)
    
    while 1:
      yield
      if self.character.current_action == ACTION_STOP_MOVING:
        #if random.random() < 0.5: self.character.send_action(ACTION_STRIKE_ONE)
        #else: break
        break
        
    self.character.send_action(ACTION_STOP_MOVING)
    
    #if random.random() < 0.5:
    #  for i in range(60): yield
      

