# -*- coding: utf-8 -*-
# Arkanae III
# Copyright (C) 2008-2009 Jean-Baptiste LAMY
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import sys, struct, math, random, copy, os
import cerealizer
import arkanae3, arkanae3.globdef as globdef

if globdef.DRIVER == "3d":
  import arkanae3.tofu.tofu4soya as tofu
else:
  import arkanae3.tofu as tofu

from arkanae3.tofu.sides import *
from arkanae3.rooms import Room


class States(object):
  def __init__(self, **kargs):
    self.__dict__.update(kargs)
  def __getattr__(self, attr):
    return 0
  def __delattr__(self, attr):
    try: object.__delattr__(self, attr)
    except: pass
    
cerealizer.register(States)


class BaseMainLoop(tofu.MainLoop):
  pass
    

class PlayerID(tofu.PlayerID):
  @classmethod
  def loads(Class, s):
    self = tofu.PlayerID.loads(s)
    return self
  
  
UNIVERSE_DATA = None
  
class UniverseData(object):
  def __new__(Class):
    new = object.__new__(Class)
    
    global UNIVERSE_DATA
    UNIVERSE_DATA = new
    
    return new
    
  def __init__(self):
    self.path            = ""
    self.states          = States(
      chapitre = 0,
      )
    self.level_states    = {}
    self.items_on_ground = {}
    
  @classmethod
  def load(Class):
    filename = os.path.join(tofu.SAVED_GAME_DIR, tofu.GAME, "universe.data")
    if os.path.exists(filename):
      print "* Arkanae 3 * Load universe from %s." % filename
      return cerealizer.load(open(filename, "rb"))
    else:
      print "* Arkanae 3 * New universe."
      return Class()
    
  def save(self):
    filename = os.path.join(tofu.SAVED_GAME_DIR, tofu.GAME, "universe.data")
    print "* Arkanae 3 * Save universe in %s." % filename
    cerealizer.dump(self, open(filename, "wb"))
    
cerealizer.register(UniverseData)

class Player(tofu.Player):
  def __init__(self, player_id):
    if tofu.has_side("single"):
      self.universe_data = UniverseData()
      
    tofu.Player.__init__(self, player_id)
    
    character = self.mobile = arkanae3.game.Hero()
    if globdef.FIRST_ROOM:
      level   = arkanae3.game.Level.get("%s_%s" % globdef.FIRST_ROOM)
      character.i = 3.5
      character.j = 2.5
    else:
      level   = arkanae3.game.Level.get("0_0")
      character.i = 1.5
      character.j = 0.5
    character.level = level
    character.set_display_name(self.filename)
    self.add_mobile(character)
    
  def killed(self):
    self.logout(0)
    
  def login(self, sock, udp_address):
    
    if not self.mobiles:
      self.mobile.level = arkanae3.game.Level.get("0_0") # XXX selon chapitre
      self.mobile.i = 3.5
      self.mobile.j = 2.5
      self.mobile.angle = 180.0
      self.mobile.current_action = ACTION_STOP_MOVING
      
      self.mobile.life = self.mobile.max_life
      self.mobile.mana = self.mobile.max_mana
      self.add_mobile(self.mobile)
      
    tofu.Player.login(self, sock, udp_address)
    
    # In single mode, remove monster in the current room !
    if tofu.has_side("single"):
      for mobile in self.mobile.level.mobiles[:]:
        if isinstance(mobile, BaseMonster) and (mobile.get_relation(self.mobile) == -1):
          self.mobile.level.remove_mobile(mobile)
          
  def logout(self, save = None):
    if save is None:
      if tofu.has_side("single"): save = 0
      else:                       save = 1
    tofu.Player.logout(self, save)
    
  def save(self):
    self.mobile.level.save()
    tofu.Player.save(self)
    
          
class BaseLevel(tofu.Level):
  def __init__(self, filename):
    tofu.Level.__init__(self)
    
    I, J = filename.split("_")
    self.I          = int(I)
    self.J          = int(J)
    self.filename   = filename
    
    # XXX create a random level
    print "* Arkanae 3 * New level %s..." % filename
    
    self.room = Room.get(self.I, self.J)
    
    if tofu.has_side("server"):
      self.add_items_on_ground() # Cannot do that in prepare, because prepare is called after sending the level to the client
      
  @side("single")
  def prepare(self):
    self.room.prepare(self)
    
    self.add_items_on_ground() # Cannot do that in __init__, because UNIVERSE_DATA is not loaded yet for the first level
    
  def add_items_on_ground(self):
    data = UNIVERSE_DATA.items_on_ground.get(self.filename)
    if data:
      items_on_ground = cerealizer.loads(data)
      for item_on_ground in items_on_ground: item_on_ground.loaded()

  @side("single", "server")
  def set_active(self, active):
    old_active = self.active
    
    tofu.Level.set_active(self, active)
    
    if active and not old_active: self.prepare()
    
  @side("single", "server")
  def save(self):
    items_on_ground = [mobile for mobile in self.mobiles if isinstance(mobile, BaseItemOnGround)]
    if items_on_ground:
      UNIVERSE_DATA.items_on_ground[self.filename] = cerealizer.dumps(items_on_ground)
    else:
      if UNIVERSE_DATA.items_on_ground.has_key(self.filename):
        del UNIVERSE_DATA.items_on_ground[self.filename]
    #pass # Never save level
    #tofu.Level.save(self)
    
  @side("client")
  def save(self):
    pass
  
  def __getstate__(self):
    return (self._filename, self.uid, self.round, self.active, self.mobiles, self.I, self.J)
  
  def __setstate__(self, state):
    self._filename, self.uid, self.round, self.active, self.mobiles, self.I, self.J = state
    self.room          = Room.get(self.I, self.J)
    self.state_counter = 0
    
ACTION_MOVE_LEFT         = "<"
ACTION_MOVE_LEFT_DOWN    = "1"
ACTION_MOVE_LEFT_UP      = "7"
ACTION_MOVE_RIGHT        = ">"
ACTION_MOVE_RIGHT_DOWN   = "3"
ACTION_MOVE_RIGHT_UP     = "9"
ACTION_MOVE_UP           = "^"
ACTION_MOVE_DOWN         = "v"
ACTIONS_MOVE = set([
  ACTION_MOVE_LEFT,
  ACTION_MOVE_LEFT_DOWN,
  ACTION_MOVE_LEFT_UP,
  ACTION_MOVE_RIGHT,
  ACTION_MOVE_RIGHT_DOWN,
  ACTION_MOVE_RIGHT_UP,
  ACTION_MOVE_UP,
  ACTION_MOVE_DOWN,
])
ACTION_TURN_LEFT         = "("
ACTION_TURN_RIGHT        = ")"
ACTION_GO_FRONT          = "-"
ACTION_GO_BACK           = "_"
ACTION_STOP_MOVING       = " "
ACTION_GOTO              = "*"
ACTION_GOTO_STRIKE       = "%"
ACTION_LOOK_AT           = "~"

ACTIONS_GOTO = set([
  ACTION_GOTO,
  ACTION_GOTO_STRIKE,
  ACTION_LOOK_AT,
  ])

ACTION_STRIKE            = "/"
ACTION_STRIKE_ONE        = "|"
ACTIONS_STRIKE = set([
  ACTION_STRIKE,
  ACTION_STRIKE_ONE,
])


ACTION_HIT               = ","
ACTION_HIT_BACK          = ";"
ACTION_DIE               = "."
ACTION_FLEE              = "F"
ACTION_CHAT              = "C"
ACTION_SAY               = "s"
ACTION_EQUIP             = "E"
ACTION_UNEQUIP           = "e"
ACTION_DROP              = "d"
ACTION_DROP_ALL          = "D"
ACTION_USE               = "U"
ACTION_GRAB              = "G"
ACTION_SET_KEY           = "K"
ACTION_DISCU             = "B"
ACTION_ANSWER            = "a"
ACTION_ACTIVATE          = "Z"
ACTION_CAST              = "c"

ACTION_EXPLODE           = "X"

UNCANCELABLE_ACTIONS = set([ACTION_HIT, ACTION_HIT_BACK, ACTION_DIE, ACTION_FLEE, ACTION_CAST])

MESSAGE_LIFE             = "L"
MESSAGE_MANA             = "M"
MESSAGE_RESISTANCES      = "R"
MESSAGE_CHAT             = "C"
MESSAGE_DISCU            = "B"
MESSAGE_SAY              = "s"
MESSAGE_EQUIPED          = "E"
MESSAGE_UNEQUIPED        = "e"
MESSAGE_USED             = "U"
MESSAGE_DROPPED          = "d"
MESSAGE_DROPPED_ALL      = "D"
MESSAGE_GRABBED          = "G"
MESSAGE_GAINED           = "g"
MESSAGE_LOST             = "l"
MESSAGE_ASK              = "A"
MESSAGE_CASTED           = "c"
MESSAGE_SOUND            = "S"

MESSAGE_OPENED           = "O"
MESSAGE_CLOSED           = "-"
MESSAGE_EXPLODE          = "X"

ATTACK_BASH     = 0
ATTACK_SHARP    = 1
ATTACK_LIGHTING = 2
ATTACK_FIRE     = 3
ATTACK_HEAL     = 9

ATTACK_RANGE_SHORT = 1.1
ATTACK_RANGE_LONG  = 1.4
ATTACK_RANGE_ALL   = 100.0

#ANGLE_SPEED = 22.75
ANGLE_SPEED  = 22.5
CHAR_RADIUS  = 0.35
SHOOT_RADIUS = 0.2
MOVE_DATA = {
  # ACTION_NAME:           (angle, speed_i, speed_j)
  ACTION_MOVE_UP:          (  0,  0.0,  1.0),
  ACTION_MOVE_LEFT_UP:     ( 45, -0.7,  0.7),
  ACTION_MOVE_LEFT:        ( 90, -1.0,  0.0),
  ACTION_MOVE_LEFT_DOWN:   (135, -0.7, -0.7),
  ACTION_MOVE_DOWN:        (180,  0.0, -1.0),
  ACTION_MOVE_RIGHT_DOWN:  (225,  0.7, -0.7),
  ACTION_MOVE_RIGHT:       (270,  1.0,  0.0),
  ACTION_MOVE_RIGHT_UP:    (315,  0.7,  0.7),
}

FACTION_HERO      = 0
FACTION_ECHASSIAN = 1
FACTION_LIGE      = 2
FACTION_FIDE      = 3
FACTION_BERSERK   = 4


FLAG_ITEM_LORE  = 1
FLAG_DODGE      = 2
FLAG_LARGE_SIZE = 4
FLAG_TRAITOR    = 8
FLAG_RAGE       = 16

NB_KEPT_LEVELS  = 1

class Strikeable(object):
  def hit(self, by): pass
  
  def get_relation(self, other): return -1
  
class Discutable(object):
  discute_script = None
  def discute(self, hero):
    if self.discute_script:
      self.discute_script(hero, self)
      return 1
    
  def get_relation(self, other): return 1
  

class Blocker(object):
  life            = 1.0
  blocking_radius = 0.7
  def block(self, blocked): return 0

class BaseInvisibleMobile(tofu.Mobile):
  i    = 0
  j    = 0
  life = 1.0
  

class BaseCharacter(tofu.Mobile, Strikeable):
  hit_sound    = "couic1.wav"
  die_sound    = "couic2.wav"
  strike_sound = "coup.wav"
  die_script   = None
  last_hit     = -1000
  liferatio    = 0.15
  discussion   = None
  k            = 0.0
  def __init__(self, model_name = None):
    super(BaseCharacter, self).__init__()
    self.i              = 3.5
    self.j              = 2.5
    self.angle          = 180.0
    self.speed_i        = 0.0
    self.speed_j        = 0.0
    self.speed_angle    = 0.0
    self.current_action = ACTION_STOP_MOVING
    self.last_noticeable_round = 0
    self.last_important_round  = 0
    self.controller     = None
    self.animation_pos  = 0
    self.goto_i         = 0.0
    self.goto_j         = 0.0
    self.goto_angle     = 0.0
    self.display_name   = u""
    
    self.max_life        = 3.0
    self.life            = 3.0
    self.items           = []
    self.flag            = 0 #FLAG_TRAITOR

    if model_name: self.model_name = model_name
    
  def loaded(self):
    for item in self.items: item.loaded()
    tofu.Mobile.loaded(self)
    self.calc_stats()
    self._init()
    
  def discard(self):
    tofu.Mobile.discard(self)
    
    for item in self.items: item.discard()
    
  def _init(self):
    self.set_model_name(self.model_name)
    if self.life <= 0.0: self.set_animation_name("mort2")
    else:                self.set_animation_name("attente")
    self.set_display_name(self.display_name)
    
  def __getstate__(self):
    return (self.uid, self.player_name, self.local, self.bot, self.level, self.items, self.flag, self.display_name, self.i, self.j, self.angle, self.current_action, self.life, self.max_life, self.goto_i, self.goto_j, self.goto_angle, self.get_model_name(), self.strength, self.endurance)
  
  def __setstate__(self, state):
    self.uid, self.player_name, self.local, self.bot, self.level, self.items, self.flag, self.display_name, self.i, self.j, self.angle, self.current_action, self.life, self.max_life, self.goto_i, self.goto_j, self.goto_angle, self.model_name, self.strength, self.endurance = state
    self.speed_i        = 0.0
    self.speed_j        = 0.0
    self.speed_angle    = 0.0
    self.last_noticeable_round = 0
    self.last_important_round  = 0
    self.controller     = None
    self.animation_pos  = 0
    self.move_started   = 0
    if self.current_action == ACTION_CAST: self.current_action = ACTION_STOP_MOVING # Spell and target are not saved
    
  def get_network_state(self):
    if self.current_action in ACTIONS_GOTO:
      if self.goto_target:
        return self.current_action + struct.pack("!fff" , self.goto_i, self.goto_j, self.goto_angle) + self.goto_target.dumpsuid() + struct.pack("!fff", self.i, self.j, self.angle)
      else:
        return self.current_action + struct.pack("!fffHfff", self.goto_i, self.goto_j, self.goto_angle, 0, self.i, self.j, self.angle)
    else:
      return self.current_action + struct.pack("!fff", self.i, self.j, self.angle)
    
  def read_network_state(self, f):
    action = f.read(1)
    if action in ACTIONS_GOTO: action += f.read(14)
    self.i, self.j, self.angle = struct.unpack("!fff", f.read(12))
    self.do_action(action)
    
  def set_controller(self, controller):
    self.controller = controller
    if controller: self.controller.character = self
    
  def generate_actions(self):
    if self.controller: self.controller.generate_actions()
    
  def do_action(self, action):
    if   action == ACTION_HIT:
      self.speed_i = self.speed_j = self.speed_angle = 0.0
      self.current_action = ACTION_HIT
      self.set_animation_name("couic")
      self.set_current_state_importance(2)
      
    elif action == ACTION_HIT_BACK:
      self.speed_i = self.speed_j = self.speed_angle = 0.0
      self.current_action = ACTION_HIT_BACK
      self.set_animation_name("couic")
      self.set_current_state_importance(2)
      
    elif action == ACTION_DIE:
      if self.current_action != ACTION_DIE:
        self.speed_i = self.speed_j = self.speed_angle = 0.0
        self.current_action = ACTION_DIE
        self.set_animation_name("mort")
        self.set_current_state_importance(2)
      
    elif action == ACTION_FLEE:
      self.current_action = ACTION_FLEE
      self.set_current_state_importance(2)
      self.speed_angle = 0.0
      
    elif action == ACTION_DROP_ALL:
      self.send_message(action)
      
    elif action.startswith(ACTION_CHAT):
      self.send_message(action)
      
    else:
      if not self.current_action in UNCANCELABLE_ACTIONS:
        if (action in ACTIONS_MOVE) or (action == ACTION_GO_FRONT) or (action == ACTION_GO_BACK) or (action == ACTION_TURN_LEFT) or (action == ACTION_TURN_RIGHT):
          self.current_action = action
          self.speed_i = self.speed_j = self.speed_angle = 0.0
          self.move_started = 0
          self.set_current_state_importance(2)
          
        elif action == ACTION_STOP_MOVING:
          #let finish the strike
          if self.current_action in ACTIONS_STRIKE:
            self.current_action = ACTION_STRIKE_ONE
          else:
            self.current_action = ACTION_STOP_MOVING
            self.speed_i = self.speed_j = self.speed_angle = 0.0
            self.set_animation_name("attente")
          self.set_current_state_importance(2)
          
        elif action in ACTIONS_STRIKE:
          if (self.current_action != ACTION_STRIKE) and (self.current_action != ACTION_STRIKE_ONE):
            self.speed_i = self.speed_j = self.speed_angle = 0.0
            self.set_animation_name("frappe")
            self.set_current_state_importance(2)
          self.current_action = action
          
        elif action[0] in ACTIONS_GOTO:
          self.current_action = action[0]
          self.speed_i = self.speed_j = self.speed_angle = 0.0
          self.goto_i, self.goto_j, self.goto_angle = struct.unpack("!fff", action[1:13])
          self.goto_target = tofu.Unique.undumpsuid(action[13:])
          self.set_current_state_importance(2)
          
        elif action.startswith(ACTION_USE):
          item = tofu.Unique.undumpsuid(action[1:])
          if (isinstance(item, Usable) or isinstance(item, Limited)) and (item in self.items):
            if item.usable_by(self) or ((self.flag & FLAG_ITEM_LORE) and self.use_power(arkanae3.game.ItemLore)): self.send_message(action)
            
        elif action.startswith(ACTION_EQUIP):
          item = tofu.Unique.undumpsuid(action[1:])
          if isinstance(item, Equipable) and (item in self.items):
            if item.usable_by(self) or ((self.flag & FLAG_ITEM_LORE) and self.use_power(arkanae3.game.ItemLore)): self.send_message(action)
            
        elif action.startswith(ACTION_UNEQUIP):
          item = tofu.Unique.undumpsuid(action[1:])
          if isinstance(item, Equipable) and (item in self.items):
            self.send_message(action)
            
        elif action.startswith(ACTION_DROP):
          item = tofu.Unique.undumpsuid(action[1:])
          if isinstance(item, Dropable) and (item in self.items):
            if isinstance(item, Equipable) and item.equiped:
              self.send_message(ACTION_UNEQUIP + action[1:])
            self.send_message(action)
            
        elif action.startswith(ACTION_GRAB):
          item_on_ground = tofu.Unique.undumpsuid(action[1:])
          if item_on_ground:
            if isinstance(item_on_ground, BaseItemOnGround) and (abs(item_on_ground.i - self.i) < 1.5) and (abs(item_on_ground.j - self.j) < 1.5):
              self.send_message(action)
              
        elif action.startswith(ACTION_CAST):
          if not tofu.has_side("client"):
            self.casted_spell = tofu.Unique.undumpsuid(action[1:3])
            self.spell_target = tofu.Unique.undumpsuid(action[3:5])
          self.current_action = ACTION_CAST
          self.speed_i = self.speed_j = self.speed_angle = 0.0
          self.set_animation_name("sort")
          self.set_current_state_importance(2)
          
        elif action.startswith(ACTION_SET_KEY):
          item = tofu.Unique.undumpsuid(action[3:5])
          if item in self.items:
            self.set_key(struct.unpack("!H", action[1:3])[0], item)
            
  def can_pass(self, speed_i, speed_j):
    if   speed_i > 0: collid_i = self.i + CHAR_RADIUS
    elif speed_i < 0: collid_i = self.i - CHAR_RADIUS
    else:             collid_i = self.i
    if   speed_j > 0: collid_j = self.j + CHAR_RADIUS
    elif speed_j < 0: collid_j = self.j - CHAR_RADIUS
    else:             collid_j = self.j
    bloc = self.level.room.blocs[int(collid_i + 1.0)][int(collid_j + 1.0)]
    
    if   bloc  > 3: return 0
    elif bloc == 1:
      self.change_room(1); return 0
    elif bloc == 2:
      if not((-0.51 < collid_i < 7.51) and (-0.51 < collid_j < 5.51)):
        self.change_room(0); return 0
        
    i = self.i + speed_i
    j = self.j + speed_j
    for mobile in self.level.mobiles:
      if (not mobile is self) and isinstance(mobile, Blocker) and (mobile.life > 0.0):
        if (abs(mobile.i - i) < mobile.blocking_radius) and (abs(mobile.j - j) < mobile.blocking_radius):
          if mobile.block(self) == 0: return 0
    return 1
  
  def change_room(self, special): return 0
  
  
  def do_physics(self):
    if   self.current_action == ACTION_STOP_MOVING: return
    elif self.current_action in MOVE_DATA:
      angle, speed_i, speed_j = MOVE_DATA[self.current_action]
      rel_angle = (self.angle - angle) % 360
      if    20.0 <  rel_angle < 180.0:
        self.speed_angle = -ANGLE_SPEED
      elif 180.0 <= rel_angle < 340.0:
        self.speed_angle =  ANGLE_SPEED
      else:
        self.angle       =  angle
        self.speed_angle =   0.0
        
        if not self.move_started:
          self.move_started = 1
          self.set_animation_name("marche")
          #self.angle       =  angle
          self.speed_angle =   0.0

      speed_i *= self.move_speed
      speed_j *= self.move_speed

      if self.can_pass(speed_i, speed_j):
        self.speed_i = speed_i
        self.speed_j = speed_j
      else:
        self.speed_i = self.speed_j = 0
          
    elif self.current_action == ACTION_FLEE:
      di, dj = angle2vector(self.angle)
      
      if   di > 0: ci =  0.12
      elif di < 0: ci = -0.12
      else:        ci =  0.0
      if   dj > 0: cj =  0.12
      elif dj < 0: cj = -0.12
      else:        cj =  0.0
      can_pass = self.can_pass(ci, cj)
      if not can_pass:
        if abs(di) > abs(dj):
          dj = 0.0
          can_pass = self.can_pass(ci, 0.0)
        else:
          di = 0.0
          can_pass = self.can_pass(0.0, cj)
          
      if can_pass:
        self.speed_i = di * self.move_speed
        self.speed_j = dj * self.move_speed
      else:
        self.speed_i = self.speed_j = 0.0
        self.current_action = ACTION_GO_FRONT
        self.send_action(ACTION_STOP_MOVING)
        
      if self.animation_name != "marche": self.set_animation_name("marche")
      
    elif self.current_action == ACTION_GO_FRONT:
      di, dj = angle2vector(self.angle)
      
      if   di > 0: ci =  self.move_speed
      elif di < 0: ci = -self.move_speed
      else:        ci =  0.0
      if   dj > 0: cj =  self.move_speed
      elif dj < 0: cj = -self.move_speed
      else:        cj =  0.0
      can_pass = self.can_pass(ci, cj)
      if not can_pass:
        if abs(di) > abs(dj):
          dj = 0.0
          can_pass = self.can_pass(ci, 0.0)
        else:
          di = 0.0
          can_pass = self.can_pass(0.0, cj)
          
      if can_pass:
        self.speed_i = di * self.move_speed
        self.speed_j = dj * self.move_speed
      else:
        self.speed_i = self.speed_j = 0.0
        self.send_action(ACTION_STOP_MOVING)
        
      if self.animation_name != "marche": self.set_animation_name("marche")
      
    elif self.current_action == ACTION_HIT_BACK:
      if self.animation_pos == 6:
        self.current_action = ACTION_STOP_MOVING
        self.set_animation_name("attente")
        self.speed_i = self.speed_j = self.speed_angle = 0.0
      else:
        di, dj = angle2vector(self.angle + 180.0)

        if   di > 0: ci =  0.16
        elif di < 0: ci = -0.16
        else:        ci =  0.0
        if   dj > 0: cj =  0.16
        elif dj < 0: cj = -0.16
        else:        cj =  0.0
        can_pass = self.can_pass(ci, cj)
        if not can_pass:
          if abs(di) > abs(dj):
            dj = 0.0
            can_pass = self.can_pass(ci, 0.0)
          else:
            di = 0.0
            can_pass = self.can_pass(0.0, cj)

        if can_pass:
          self.speed_i = di * 0.16
          self.speed_j = dj * 0.16
        else:
          self.speed_i = self.speed_j = 0.0
        
    elif self.current_action == ACTION_TURN_LEFT:
      self.speed_angle = -11.25 #-11.375
      
    elif self.current_action == ACTION_TURN_RIGHT:
      self.speed_angle =  11.25 #11.375
      
    elif self.current_action == ACTION_HIT:
      if self.animation_pos == 6:
        self.current_action = ACTION_STOP_MOVING
        self.set_animation_name("attente")
        
    elif self.current_action == ACTION_DIE:
      if self.animation_name == "mort":
        if self.animation_pos == 9:
          self.set_animation_name("mort2")
      else:
        self.life -= 0.01
        if self.life < -1.0:
          if tofu.has_side("server") or tofu.has_side("single"):
            self.kill()
            
    elif self.current_action == ACTION_STRIKE:
      if   self.animation_pos == 8: tofu.MAIN_LOOP.play_sound(self.strike_sound)
      
    elif self.current_action == ACTION_STRIKE_ONE:
      if   self.animation_pos == 8: tofu.MAIN_LOOP.play_sound(self.strike_sound)
      elif self.animation_pos >= 19:
        self.current_action = ACTION_STOP_MOVING
        self.set_animation_name("attente")
        
#     elif (self.current_action == ACTION_GOTO) or (self.current_action == ACTION_GOTO_STRIKE):
#       dangle = ((self.goto_angle - self.angle) % 360.0) - 180.0
#       if   -160.0 < dangle <=   0.0: self.speed_angle =  22.75
#       elif    0.0 < dangle <  160.0: self.speed_angle = -22.75
#       else:
#         di = self.goto_i - self.i
#         dj = self.goto_j - self.j
#         d = math.sqrt(di * di + dj * dj)

#         if   di > 0: ci =  self.move_speed
#         elif di < 0: ci = -self.move_speed
#         else:        ci =  0.0
#         if   dj > 0: cj =  self.move_speed
#         elif dj < 0: cj = -self.move_speed
#         else:        cj =  0.0
#         can_pass = self.can_pass(ci, cj)
#         if not can_pass:
#           if abs(di) > abs(dj):
#             dj = 0.0
#             can_pass = self.can_pass(ci, 0.0)
#           else:
#             di = 0.0
#             can_pass = self.can_pass(0.0, cj)
            
#         if (((self.current_action == ACTION_GOTO       ) and (d > 0.2)) or
#             ((self.current_action == ACTION_GOTO_STRIKE) and (d > self.damages[0] + 0.3))) and can_pass:
#           self.speed_angle = 0
#           self.angle = self.goto_angle
#           self.speed_i = di * self.move_speed / d
#           self.speed_j = dj * self.move_speed / d
#           if self.animation_name != "marche": self.set_animation_name("marche")
#         else:
#           if (self.current_action == ACTION_GOTO_STRIKE) and (d <= self.damages[0] + 0.3):
#             self.do_action(ACTION_STRIKE_ONE)
#           else:
#             self.current_action = ACTION_STOP_MOVING
#             self.speed_i = self.speed_j = self.speed_angle = 0.0
#             if self.animation_name != "attente": self.set_animation_name("attente")
            
    elif self.current_action == ACTION_GOTO:
      dangle = ((self.goto_angle - self.angle) % 360.0) - 180.0
      if   -160.0 < dangle <=   0.0: self.speed_angle =  22.75
      elif    0.0 < dangle <  160.0: self.speed_angle = -22.75
      else:
        di = self.goto_i - self.i
        dj = self.goto_j - self.j
        d = math.sqrt(di * di + dj * dj)

        if   di > 0: ci =  self.move_speed
        elif di < 0: ci = -self.move_speed
        else:        ci =  0.0
        if   dj > 0: cj =  self.move_speed
        elif dj < 0: cj = -self.move_speed
        else:        cj =  0.0
        can_pass = self.can_pass(ci, cj)
        if not can_pass:
          if abs(di) > abs(dj):
            dj = 0.0
            can_pass = self.can_pass(ci, 0.0)
          else:
            di = 0.0
            can_pass = self.can_pass(0.0, cj)
            
        if (self.current_action == ACTION_GOTO) and (d > 0.2) and can_pass: # current_action can be different than ACTION_GOTO, if the character has changed to a different room (in can_pass)
          self.speed_angle = 0
          self.angle = self.goto_angle
          self.speed_i = di * self.move_speed / d
          self.speed_j = dj * self.move_speed / d
          if self.animation_name != "marche": self.set_animation_name("marche")
        else:
          self.current_action = ACTION_STOP_MOVING
          self.speed_i = self.speed_j = self.speed_angle = 0.0
          if self.animation_name != "attente": self.set_animation_name("attente")
          
    elif self.current_action == ACTION_GOTO_STRIKE:
      #if self.goto_target and (self.level.round % 10) == 0:
      if self.goto_target:
        self.goto_i     = self.goto_target.i
        self.goto_j     = self.goto_target.j
        self.goto_angle = vector2angle(self.goto_target.i - self.i, self.goto_target.j - self.j)
        
      dangle = ((self.goto_angle - self.angle) % 360.0) - 180.0
      if   -160.0 < dangle <=   0.0: self.speed_angle =  22.75
      elif    0.0 < dangle <  160.0: self.speed_angle = -22.75
      else:
        di = self.goto_i - self.i
        dj = self.goto_j - self.j
        d = math.sqrt(di * di + dj * dj)
        
        if   di > 0: ci =  self.move_speed
        elif di < 0: ci = -self.move_speed
        else:        ci =  0.0
        if   dj > 0: cj =  self.move_speed
        elif dj < 0: cj = -self.move_speed
        else:        cj =  0.0
        can_pass = self.can_pass(ci, cj)
        if not can_pass:
          if abs(di) > abs(dj):
            dj = 0.0
            can_pass = self.can_pass(ci, 0.0)
          else:
            di = 0.0
            can_pass = self.can_pass(0.0, cj)
            
        if (d > self.damages[0] + 0.3) and can_pass:
          self.speed_angle = 0
          self.angle = self.goto_angle
          self.speed_i = di * self.move_speed / d
          self.speed_j = dj * self.move_speed / d
          if self.animation_name != "marche": self.set_animation_name("marche")
        elif d <= self.damages[0] + 0.3:
          self.do_action(ACTION_STRIKE_ONE)
        else:
          self.current_action = ACTION_STOP_MOVING
          self.speed_i = self.speed_j = self.speed_angle = 0.0
          if self.animation_name != "attente": self.set_animation_name("attente")
          
    elif self.current_action == ACTION_LOOK_AT:
      dangle = ((self.goto_angle - self.angle) % 360.0) - 180.0
      if   -160.0 < dangle <=   0.0: self.speed_angle =  22.75
      elif    0.0 < dangle <  160.0: self.speed_angle = -22.75
      else:
        self.angle = self.goto_angle
        self.speed_i = self.speed_j = self.speed_angle = 0.0
        
    elif self.current_action == ACTION_CAST:
      if self.animation_pos >= 19:
        self.current_action = ACTION_STOP_MOVING
        self.set_animation_name("attente")
        
  def hit(self, by):
    if self.life > 0.0:
      if self.get_relation(by) != -1: return # Hit by a friend
      
      # min(self.life, self.max_life) because the player may have unequiped his armor
      if (by.flag & FLAG_RAGE) and (by.life < by.max_life / 2.0):
        life = min(self.life, self.max_life) - self.resistances[by.damages[1]] * by.damages[2] * self.liferatio * 2.0
      else:
        life = min(self.life, self.max_life) - self.resistances[by.damages[1]] * by.damages[2] * self.liferatio
        
      self.angle = (by.angle + 180.0) % 360.0
      
      if isinstance(by, BaseHero):
        if isinstance(by.weapon, Usable):
          by.send_message(MESSAGE_USED + by.weapon.dumpsuid())
          
      if life <= 0.0:
        life = 0.0
        self.send_action(ACTION_DIE)
        self.die(by)
        
      else:
        if self.last_hit > self.level.round - 30:
          if self.flag & FLAG_LARGE_SIZE:
            self.set_current_state_importance(2) # self.angle has been modified
          else:
            self.send_action(ACTION_HIT_BACK)
        else:
          if self.flag & FLAG_LARGE_SIZE:
            self.set_current_state_importance(2) # self.angle has been modified
          else:
            self.send_action(ACTION_HIT)
            
      self.send_message(MESSAGE_LIFE + str(by.damages[1]) + struct.pack("!f", life))
      
      self.last_hit = self.level.round

  def die(self, by):
    pass
  
  def do_collisions(self):
    if ((self.current_action == ACTION_STRIKE) or (self.current_action == ACTION_STRIKE_ONE)) and (self.animation_pos == 10):
      if   self.damages[0] == ATTACK_RANGE_SHORT:
        i, j = angle2vector(self.angle, ATTACK_RANGE_SHORT)
        radius = 0.25
        i += self.i
        j += self.j
        for mobile in self.level.mobiles:
          # Equivalent to (but faster):
          #if (not mobile is self) and isinstance(mobile, Strikeable) and (math.sqrt((mobile.i - i) ** 2 + (mobile.j - j) ** 2) < 0.5):
          if (not mobile is self) and isinstance(mobile, Strikeable) and ((mobile.i - i) ** 2 + (mobile.j - j) ** 2 < radius):
            mobile.hit(self)
            
      elif self.damages[0] == ATTACK_RANGE_LONG:
        mobiles = set()
        i, j = angle2vector(self.angle, ATTACK_RANGE_SHORT)
        radius = 0.25
        i += self.i
        j += self.j
        for mobile in self.level.mobiles:
          if (not mobile is self) and isinstance(mobile, Strikeable) and ((mobile.i - i) ** 2 + (mobile.j - j) ** 2 < radius):
            mobiles.add(mobile)
        i, j = angle2vector(self.angle, ATTACK_RANGE_LONG - 0.1)
        radius = 0.30
        i += self.i
        j += self.j
        for mobile in self.level.mobiles:
          if (not mobile is self) and isinstance(mobile, Strikeable) and ((mobile.i - i) ** 2 + (mobile.j - j) ** 2 < radius):
            mobiles.add(mobile)
        for mobile in mobiles: mobile.hit(self)
        
      elif self.damages[0] == ATTACK_RANGE_ALL:
        for mobile in self.level.mobiles:
          if (not mobile is self) and isinstance(mobile, Strikeable):
            mobile.hit(self)
            
    elif (self.current_action == ACTION_CAST) and (self.animation_pos == 12):
      #self.send_message(MESSAGE_CASTED + self.casted_spell.dumpsuid() + self.spell_target.dumpsuid())
      self.casted_spell.casted(self, self.spell_target)
      
  def advance_time(self, proportion):
    tofu.Mobile.advance_time(self, proportion)
    
    if self.speed_angle:
      self.angle = (self.angle + self.speed_angle * proportion) % 360.0
      
    self.i += self.speed_i * proportion
    self.j += self.speed_j * proportion
    
  def set_display_name(self, name):
    self.display_name = name
    
  def do_message(self, message):
    if   message.startswith(MESSAGE_LIFE):
      old_life = self.life
      self.life = struct.unpack("!f", message[2:])[0]
      
      if old_life > self.life:
        if self.life:
          if (self.flag & FLAG_RAGE) and (old_life >= 0.5 * self.max_life) and (self.life < 0.5 * self.max_life):
            tofu.MAIN_LOOP.play_sound("grogne1.wav")
          else:
            tofu.MAIN_LOOP.play_sound(self.hit_sound)
        else:
          tofu.MAIN_LOOP.play_sound(self.die_sound)
      effect = int(message[1])
      if effect >= 0: self.create_effect(effect)
      
    elif message.startswith(MESSAGE_CASTED):
      casted_spell = tofu.Unique.undumpsuid(message[1:3])
      spell_target = tofu.Unique.undumpsuid(message[3:5])
      casted_spell.casted(self, spell_target)
    elif message.startswith(MESSAGE_EQUIPED):
      if self.local: tofu.MAIN_LOOP.play_sound("menu1.wav")
      item = tofu.Unique.undumpsuid(message[1:])
      item.set_equiped(self, 1)
    elif message.startswith(MESSAGE_UNEQUIPED):
      if self.local: tofu.MAIN_LOOP.play_sound("menu1.wav")
      item = tofu.Unique.undumpsuid(message[1:])
      item.set_equiped(self, 0)
    elif message.startswith(MESSAGE_USED):
      item = tofu.Unique.undumpsuid(message[1:])
      item.used(self)
    elif message.startswith(MESSAGE_DROPPED):
      if self.local: tofu.MAIN_LOOP.play_sound("depose.wav")
      item = tofu.Unique.undumpsuid(message[1:])
      item.dropped(self)
    elif message.startswith(MESSAGE_GRABBED):
      item_on_ground = tofu.Unique.undumpsuid(message[1:])
      if item_on_ground.level:
        if self.local: tofu.MAIN_LOOP.play_sound("menu1.wav")
        if isinstance(item_on_ground.item, Combinable):
          for item in self.items:
            if (item.__class__ is item_on_ground.item.__class__) or ():
              item.combine_with(item_on_ground.item)
              break
          else: self.add_item(item_on_ground.item)
        else: self.add_item(item_on_ground.item)
        item_on_ground.item = None # Needed, because without this, the remove_mobile would discard the item_on_ground, and thus the item itself !
        if tofu.has_side("server") or tofu.has_side("single"):
          self.level.remove_mobile(item_on_ground)
          
    elif message.startswith(MESSAGE_GAINED):
      power = cerealizer.loads(message[1:])
      tofu.Unique._alls[power.uid] = power # Hack ! Needed, so as the server assign this UID to the object, without creating a new one.
      self.add_power(power)
    elif message.startswith(MESSAGE_LOST):
      item = tofu.Unique.undumpsuid(message[1:])
      self.remove_item(item)
    elif message == MESSAGE_DROPPED_ALL:
      if isinstance(self, BaseMonster):
        if self.bonus and (tofu.has_side("server") or tofu.has_side("single")):
          self.level.add_mobile(arkanae3.game.ItemOnGround(self.bonus, self.i, self.j))
      else:
        to_drops = [item for item in self.items if isinstance(item, Dropable)]
        if to_drops:
          angle  = 0.0
          dangle = 6.2834 / len(to_drops)
          for item in to_drops:
            item.dropped(self, self.i + 0.7 * math.cos(angle), self.j + 0.7 * math.sin(angle))
            angle += dangle
            
  def goto(self, i, j):
    angle = vector2angle(i - self.i, j - self.j)
    self.send_action(ACTION_GOTO + struct.pack("!fffH", i, j, angle, 0))
    
  def has_item(self, Item):
    for item in self.items:
      if isinstance(item, Item): return item
      
  def add_item(self, item):
    for i in range(len(self.items)):
      if not isinstance(self.items[i], Item):
        self.items.insert(i, item)
        break
    else: self.items.append(item)
    item.gained(self)
    
  def add_power(self, power):
    # Remove power included in the new one (e.g. Heavy Basher 1 when you get the 2).
    for item in self.items[:]:
      if item.__class__ in power.deprecates:
        self.remove_item(item)
        
    self.items.append(power)
    power.gained(self)
    
  def remove_item(self, item):
    if item in self.items: # Remove_item can be called multiple time, e.g. when a Limited has 0 use left, and is equiped.
      item.lost(self)
      if item in self.items: # Lost may remove the item...
        self.items.remove(item)
  remove_power = remove_item
  
  def use_power(self, Power):
    for item in self.items:
      if isinstance(item, Power):
        self.use(item)
        return 1
    
  def use    (self, item          ): self.send_action(ACTION_USE     + item          .dumpsuid())
  def equip  (self, item          ): self.send_action(ACTION_EQUIP   + item          .dumpsuid())
  def unequip(self, item          ): self.send_action(ACTION_UNEQUIP + item          .dumpsuid())
  def drop   (self, item          ): self.send_action(ACTION_DROP    + item          .dumpsuid())
  def grab   (self, item_on_ground): self.send_action(ACTION_GRAB    + item_on_ground.dumpsuid())
  
  def kill(self):
    self.send_action(ACTION_DROP_ALL)
    
    tofu.Mobile.kill(self)
    
  def calc_stats(self):
    self.weapon = self.armor = None
    for item in self.items:
      if   isinstance(item, Weapon) and item.equiped: self.weapon = item
      elif isinstance(item, Armor ) and item.equiped: self.armor  = item
      
    if self.weapon:
      self.strike_sound = self.weapon.strike_sound
      self.damages = list(self.weapon.damages)
      self.damages[2] += self.strength
    else:
      self.strike_sound = "coup.wav"
      self.damages = [ATTACK_RANGE_SHORT, ATTACK_BASH, self.strength]
      
    if self.armor:
      self.max_life = self.armor.protection + self.endurance
      self.resistances = [self.__class__.resistances[i] + self.armor.resistance[i] for i in range(4)]
    else:
      self.max_life = self.endurance
      self.resistances = list(self.__class__.resistances)
      
    for item in self.items: item.calc_stats(self)
    
  def get_relation(self, character):
    return min(self.faction_relations[character.faction], character.faction_relations[self.faction])
  

class BaseMonster(BaseCharacter, Discutable, Blocker):
  non_targetable = 0
  strength       = 2
  endurance      = 3
  move_speed     = 0.09
  def __init__(self, model_name = None):
    super(BaseMonster, self).__init__(model_name)
    
    self.bot = 1
    import arkanae3.ai
    self.set_controller(getattr(arkanae3.ai, "%sController" % self.__class__.__name__)())
    
  def loaded(self):
    BaseCharacter.loaded(self)
    
    if self.local:
      import arkanae3.ai
      self.set_controller(getattr(arkanae3.ai, "%sController" % self.__class__.__name__)())
      
  def __getstate__(self):
    return BaseCharacter.__getstate__(self) + (self.state_id, self.discute_script, self.die_script, self.bonus)
  
  def __setstate__(self, state):
    self.state_id, self.discute_script, self.die_script, self.bonus = state[-4:]
    BaseCharacter.__setstate__(self, state[:-4])
    
  def prepare(self, level, state_id, i, j, die_script = None, endurance = None, strength = None, items = None, faction = None, discute_script = None, angle = 180.0, bonus = None, life = None):
    if state_id and UNIVERSE_DATA.level_states.get((level.I, level.J, state_id), 0):
      # Monster is dead !
      return
    
    self.state_id       = state_id
    #self.opened         = UNIVERSE_DATA.level_states.get((level.I, level.J, state_id), 0)
    self.i              = i
    self.j              = j
    self.angle          = angle
    self.die_script     = die_script
    self.discute_script = discute_script
    self.bonus          = bonus
    level.add_mobile(self)
    
    if endurance: self.endurance = endurance
    if strength : self.strength  = strength

    if not faction is None: self.faction = faction
    
    if items:
      self.items = items
      for item in self.items: item.set_equiped(self, 1)
      
    self.calc_stats()
    if life: self.life = life
    else:    self.life = self.max_life
    
    self._init()
    return self
  
  def die(self, by):
    BaseCharacter.die(self, by)
    
    if isinstance(by, BaseShoot):
      by = by.caster
      
    if self.die_script and isinstance(by, BaseHero): self.die_script(by)
    
    if self.state_id:
      UNIVERSE_DATA.level_states[self.level.I, self.level.J, self.state_id] = 1 # 0 living, 1 dead
    
  def discute(self, hero):
    super(BaseMonster, self).discute(hero)
    
    angle = vector2angle(hero.i - self.i, hero.j - self.j)
    self.send_action(ACTION_LOOK_AT + struct.pack("!fffH", hero.i, hero.j, angle, 0))
    


class BaseHero(BaseCharacter, Blocker):
  non_targetable    = 0
  faction           = FACTION_HERO
  faction_relations = [1, 0, 0, 0, -1]
  liferatio         = 0.15
  
  def __init__(self):
    super(BaseHero, self).__init__()
    
    self.current_script  = 0
    
    self.max_item_weight = 1
    self.intelligence    = 0
    self.strength        = 2
    self.endurance       = 3
    self.max_mana        = 0.0
    self.mana            = 0.0
    self.spcs            = States(
      chapitre = 0,
#      "" : ("0_0", 3.5, 2.5),
      )
    self.kept_levels     = []
    
    if globdef.FIRST_ROOM:
      self.items       = [arkanae3.game.TeleportationPotion(), arkanae3.game.LighteningWiz(), arkanae3.game.HealingSpell(), arkanae3.game.ManaPotion(), arkanae3.game.LifePotion(), arkanae3.game.Robe(), arkanae3.game.Plastron(), arkanae3.game.Club(), arkanae3.game.Knife(), arkanae3.game.Staff(), arkanae3.game.Lance()]
      self.intelligence    = 4.0
      self.max_mana        = 4.0
      self.mana            = 4.0
      self.spcs.miniquete_nego = 10
      self.spcs.miniquete_bateau = 10
      self.spcs.miniquete_monstre = 10
      
    self.keys        = {}
    self._init()
    self.life        = 1.0
    for item in self.items: item.gained(self)
    
  def __getstate__(self):
    return BaseCharacter.__getstate__(self) + (self.keys, self.max_item_weight, self.intelligence, self.mana, self.max_mana, self.spcs)
  
  def __setstate__(self, state):
    self.keys, self.max_item_weight, self.intelligence, self.mana, self.max_mana, self.spcs = state[-6:]
    self.discussion     = None
    self.current_script = 0
    self.kept_levels    = []
    BaseCharacter.__setstate__(self, state[:-6])
    
  def calc_stats(self):
    self.max_mana = self.intelligence
    
    self.faction_relations = list(self.__class__.faction_relations)
    
    BaseCharacter.calc_stats(self)
    
    #if self.mana > self.max_mana: self.mana = self.max_mana
    
    self.update_life()
    
  def begin_round(self):
    super(BaseHero, self).begin_round()
    
    if self.non_targetable: self.non_targetable -= 1
    
  @side("single", "server")
  def change_room(self, special):
    for mobile in self.level.mobiles:
      if isinstance(mobile, BaseMonster) and mobile.bot and (mobile.life > 0.0) and (mobile.controller.target is self) and (abs(mobile.i - self.i) < 3.0) and (abs(mobile.j - self.j) < 3.0):
        self.send_action(ACTION_STOP_MOVING)
        self.chat(_(u"Finissez le combat en cours !"))
        return 0
      
    if tofu.has_side("server"):
      self.send_action(ACTION_STOP_MOVING)
      
    elif self.current_action in ACTIONS_GOTO:
      self.send_action(ACTION_STOP_MOVING)
      
    if abs(self.i - 3.5) > abs(self.j - 2.5) + 1.0:
      dJ = 0
      if self.i < 3.5: dI = -1 
      else:            dI = +1
    else:
      dI = 0
      if self.j < 2.5: dJ = -1
      else:            dJ = +1

    self.i = min(7.5, max(-0.5, self.i - 8.0 * dI))
    self.j = min(5.5, max(-0.5, self.j - 6.0 * dJ))
    
    if special:
      self.i += 2.0 * dI
      self.j += 2.0 * dJ
      
    new_level = arkanae3.game.Level.get("%s_%s" % (self.level.I + dI, self.level.J + dJ))
    
    if self.kept_levels:
      print self.kept_levels, new_level, new_level is self.kept_levels[0]
      
    self.kept_levels.append(self.level)
    if len(self.kept_levels) > NB_KEPT_LEVELS:
      del self.kept_levels[0]
      
    self.level.remove_mobile(self)
    new_level.add_mobile(self)
    
    self.non_targetable = 25
    self.last_hit       = -1000
    
    if tofu.has_side("single"):
      tofu.MAIN_LOOP.reset()
      
    return 1
  
  def chat(self, text):
    self.send_action(ACTION_CHAT + text.encode("utf8"))
    
  @side("single", "server")
  def ask(self, text, options):
    text += u"\n\n" + u"\n".join(options)
    self.send_message(MESSAGE_ASK + text.encode("utf8"))
    
  @side("single", "client")
  def answer(self, option):
    self.send_action(ACTION_ANSWER + str(option))
    
  def start_discussion(self, npc):
    self.send_action(ACTION_DISCU + tofu.Unique.dumpsuid(npc))
    
    angle = vector2angle(npc.i - self.i, npc.j - self.j)
    self.send_action(ACTION_LOOK_AT + struct.pack("!fffH", npc.i, npc.j, angle, 0))
    
  def do_action(self, action):
    if   action.startswith(ACTION_DISCU):
      npc = tofu.Unique.undumpsuid(action[1:])
      if (abs(npc.i - self.i) < 1.5) and (abs(npc.j - self.j) < 1.5) and (npc.get_relation(self) >= 0):
        npc.discute(self)
        
    elif action.startswith(ACTION_ACTIVATE):
      item = tofu.Unique.undumpsuid(action[1:])
      if self.discussion and not isinstance(self.discussion, ToolDiscussion): return
      if (item in self.items): ToolDiscussion(self, item).set_discu("0")
        
    elif action.startswith(ACTION_ANSWER):
      option = int(action[1:])
      if self.discussion: self.discussion.validate(option)
      
    elif action.startswith(ACTION_SAY):
      self.send_message(action)
      
    else:
      super(BaseHero, self).do_action(action)
    
  def do_collisions(self):
    super(BaseHero, self).do_collisions()
    bloc = self.level.room.blocs[int(self.i + 1.0)][int(self.j + 1.0)]
    if -9 <= bloc <= -1:
      if self.current_script != -bloc:
        if self.current_script:
          self.level.room.do_script(self, self.current_script, "exit")
        self.current_script = -bloc
        self.level.room.do_script(self, self.current_script, "enter")
    elif self.current_script:
      self.level.room.do_script(self, self.current_script, "exit")
      self.current_script = 0
      
#     if -9 <= bloc <= -1:
#       if self.current_script != -bloc:
#         self.current_script = -bloc
#         self.level.room.do_script(self, self.current_script, "enter")
#     elif self.current_script:
#       self.level.room.do_script(self, self.current_script, "exit")
#       self.current_script = 0
      
  def do_message(self, message):
    if   message.startswith(MESSAGE_LIFE):
      super(BaseHero, self).do_message(message)
      self.update_life()
      
    elif message.startswith(MESSAGE_MANA):
      self.mana = struct.unpack("!f", message[1:])[0]
      self.update_life()
      
    elif message.startswith(MESSAGE_SOUND):
      tofu.MAIN_LOOP.play_sound(message[1:])
      
    else:
      super(BaseHero, self).do_message(message)
      
  def init_interface(self):
    self.update_life()
    
  def get_info(self):
    return (
      _(u"__attack__"     ) % u"%s +%s, %s"  % (_(u"__attack_%s__" % self.damages[1]), self.damages[2], _(u"__attack_range_%s__" % self.damages[0])) + u"\n" +
      _(u"__resistance__" ) % u", ".join([u"%s %i%%" % (_(u"__attack_%s__" % attack), 100 - round(100 * self.resistances[attack])) for attack in range(4)])
      )
  @side("single", "client", "server")
  def set_key(self, key, item):
    if item.key: self.keys.pop(item.key, None)
    old_item = self.keys.get(key)
    if old_item and (old_item in self.items):
      old_item.key = None
      
    self.keys[key] = item
    item.key = key
    
  @side("client")
  def set_key(self, key, item):
    self.send_action(ACTION_SET_KEY + struct.pack("!H", key) + item.dumpsuid()) # For the server (so as the key is saved on the server)
    
  def activate_tool(self, tool): self.send_action(ACTION_ACTIVATE + tool.dumpsuid())
  
  def gain(self, item):
    self.send_message(MESSAGE_GAINED + cerealizer.dumps(item))
    
    
class BaseController(object):
  def __init__(self): pass
  
  def generate_actions(self): pass




class BaseShoot(BaseCharacter):
  non_targetable = 1
  caster         = None
  strength       = 2
  endurance      = 3
  resistances    = [1.0, 1.0, 1.0, 1.0]
  
  def __init__(self, model_name):
    super(BaseShoot, self).__init__(model_name)
    
    self.bot            = 1
    self.current_action = ACTION_GO_FRONT
    
  def __getstate__(self):
    return BaseCharacter.__getstate__(self) + (self.damages,)
  
  def __setstate__(self, state):
    self.damages = state[-1]
    BaseCharacter.__setstate__(self, state[:-1])
    
  def generate_actions(self): pass
  
  def hit(self, by): pass
  
  def do_action(self, action):
    if action == ACTION_EXPLODE:
      self.send_message(action)
      
  def do_message(self, message):
    if message == MESSAGE_EXPLODE:
      self.current_action = ACTION_EXPLODE
      self.explode()
      
  def do_physics(self):
    if self.current_action == ACTION_GO_FRONT:
      if self.level and (not self.can_pass(self.speed_i, self.speed_j)) and self.local:
        self.send_action(ACTION_EXPLODE)
    
  def can_pass(self, speed_i, speed_j):
    if   speed_i > 0: collid_i = self.i + SHOOT_RADIUS
    elif speed_i < 0: collid_i = self.i - SHOOT_RADIUS
    else:             collid_i = self.i
    if   speed_j > 0: collid_j = self.j + SHOOT_RADIUS
    elif speed_j < 0: collid_j = self.j - SHOOT_RADIUS
    else:             collid_j = self.j
    
    if (collid_i < -0.5) or (collid_i > 7.5) or (collid_j < -0.5) or (collid_j > 5.5):
      return 0
    
    bloc = self.level.room.blocs[int(collid_i + 1.0)][int(collid_j + 1.0)]
    if bloc > 4: return 0
    
    i = self.i + speed_i
    j = self.j + speed_j
    
    return 1
    
  def do_collisions(self):
    if self.level:
      for mobile in self.level.mobiles:
        if (not mobile is self) and isinstance(mobile, Strikeable) and (mobile.life > 0.0) and (mobile.get_relation(self) == -1):
          if math.sqrt((mobile.i - self.i) ** 2 + (mobile.j - self.j) ** 2) <= self.damages[0]:
            mobile.hit(self)
            self.send_action(ACTION_EXPLODE)
      return 1
  
  
class BaseWiz(BaseShoot):
  def send_action(self, action):
    BaseShoot.send_action(self, action)
      
  def __init__(self, model_name):
    super(BaseWiz, self).__init__(model_name)
    
  def __getstate__(self):
    return BaseShoot.__getstate__(self) + (self.speed_i, self.speed_j, self.damages, self.faction, self.faction_relations)
  
  def __setstate__(self, state):
    BaseShoot.__setstate__(self, state[:-5])
    self.speed_i, self.speed_j, self.damages, self.faction, self.faction_relations = state[-5:]
    
  def prepare(self, level, i, j, speed_i, speed_j, damages, faction, faction_relations):
    self.i       = i
    self.j       = j
    self.speed_i = speed_i
    self.speed_j = speed_j
    self.angle   = vector2angle(speed_i, speed_j)
    self.damages = damages
    self.faction = faction
    self.faction_relations = faction_relations
    self._init()
    self.set_animation_name("attente")
    
    level.add_mobile(self)
    
  def generate_actions(self): pass
  
  def explode(self):
    self.speed_i = self.speed_j = 0
    if self.animation_name != "explose":
      self.set_animation_name("explose")
      
  def do_collisions(self):
    if self.current_action == ACTION_GO_FRONT:
      BaseShoot.do_collisions(self)

    if (self.current_action == ACTION_EXPLODE) and (self.animation_pos >= 12):
      if self.local:
        self.kill()
      
      

def angle2vector(angle, d = 1.0):
  angle /= 57.295779513082323
  return -d * math.sin(angle), d * math.cos(angle)
  
def vector2angle(di, dj):
  d = math.sqrt(di * di + dj * dj)
  if d == 0.0: d = 0.0001
  if di > 0: return 360.0 - math.acos(dj / d) * 57.295779513082323
  return math.acos(dj / d) * 57.295779513082323


class Script(object):
  def __init__(self, Discussion, discu, once):
    self.Discussion = Discussion
    self.discu      = discu
    self.once       = once
    
  def __call__(self, hero, npc = None):
    self.Discussion(hero, npc).set_discu(self.discu)
    if self.once: return -1
    
  def __getstate__(self):
    if self.Discussion.room:
      return self.Discussion.room.I, self.Discussion.room.J, self.Discussion.__name__, self.discu, self.once
    else:
      return None, self.Discussion.__module__, self.Discussion.__name__, self.discu, self.once
    
  def __setstate__(self, state):
    if state[0] is None:
      if not state[1].startswith("arkanae3"): raise StandardError("Security check: Cannot deserialize a Discussion class which is not in the 'arkanae3' package!")
      if not state[2].endswith("Discussion"): raise StandardError("Security check: Cannot deserialize a Discussion class which is not named 'XXXDiscussion'!")
      self.Discussion = getattr(sys.modules[state[1]], state[2])
    else:
      self.Discussion = Room.get(state[0], state[1]).Discussions[state[2]]
    self.discu = state[3]
    self.once  = state[4]
    
cerealizer.register(Script)


class BaseDiscussion(object):
  room = None
  def __init__(self, hero, npc = None):
    self.hero            = hero
    self.hero.discussion = self
    self.npc             = npc
    self.discu           = ""
    self.historique      = []
    
  #@classmethod
  #def to_script(Class, discu = "0", once = 0):
  #  def f(hero, npc = None):
  #    Class(hero, npc).set_discu(discu)
  #    if once: return -1
  #  return f
  
  @classmethod
  def to_script(Class, discu = "0", once = 0): return Script(Class, discu, once)
  
  def play_sound(self, sound):
    self.hero.send_message(MESSAGE_SOUND + sound)
    
  def end(self):
    self.hero.discussion = None
    
  def set_text(self, text):
    self.text = text
    
  def add_option(self, text = None, discu = None, force = 0):
    if discu is None:
      if self.discu[-1] in "012345678":
        discu = self.discu[:-1] + str(int(self.discu[-1]) + 1)
      else:
        discu = self.discu + "2"
    if discu in self.historique: return
    if text  is None: text  = getattr(self, "discu_%s" % discu).__doc__
    self.options.append((text, discu))

  def del_option(self, discu):
    setattr(self, "discu_%s" % discu, None)
    
  def validate(self, option):
    self.set_discu(self.options[option][1])
    
  def set_discu(self, discu):
    self.discu = discu
    self.historique.append(discu)
    self.text    = None
    self.options = []
    self.selected_option = 0
    if   discu.startswith("__echange_ok"): getattr(self, "discu_echange_ok")(int(discu[12:]))
    elif discu.startswith("__echange"   ): getattr(self, "discu_echange"   )(int(discu[ 9:]))
    else:
      #getattr(self, "discu_%s" % discu)()
      func = getattr(self, "discu_%s" % discu)
      if func:
        r = func()
      #  if r == 1: setattr(self, "discu_%s" % discu, None) # Disactivate the discussion
        
    if self.hero.discussion is self:
      if not self.options: self.end()
      if not self.text is None: self.update()
      
  def update(self):
    if self.options:
      options = []
      self.hero.ask(self.text, [option[0] for option in self.options])
      
      if self.hero.current_action != ACTION_STOP_MOVING:
        self.hero.send_action(ACTION_STOP_MOVING)
    else:
      self.hero.chat(self.text)
      
  def item_exchange(self, items):
    self.exhanging_items = items
    for i in range(len(items)):
      self.add_option(items[i].description(), "__echange%s" % i)
    self.add_option(_(u"Aucun"), "echange_annule")
    
  def discu_echange_annule(self):
    self.set_text(_(u"Soit."))
    self.add_options()
    
  def discu_echange(self, i):
    self.exhanging_item_choice = i
    
    #price = self.items[i].price
    #if isinstance(self.items[i], ChapterDependentPrice): price *= UNIVERSE_DATA.states["chapitre"]
    #if isinstance(self.items[i], Limited):               price *= self.items[i].nb_use
    cost = self.exhanging_items[i].get_price()
    
    items = [(item.get_price(), item) for        item in self.hero.items if not item.__class__ is self.exhanging_items[i].__class__]
    items = [(         price  , item) for price, item in items if price >= cost]
    items.sort()
    items = [ item                    for price, item in items]
    self.exhanging_items2 = items
    
    if items:
      self.set_text(_(u"Que me proposez-vous en échange ?"))
      for j in range(min(5, len(items))):
        self.add_option(items[j].description(), "__echange_ok%s" % j)
      self.add_option(_(u"J'ai changé d'avis."), "echange_annule")
    else:
      self.set_text(_(u"Je suis désolé, mais vous n'avez pas d'objet que j'accepterai d'échanger contre celui-ci."))
      self.add_options()
      
  def discu_echange_ok(self, j):
    self.set_text(_(u"Marché conclu !"))
    self.add_options()
    
    self.npc.add_item(self.exhanging_items[self.exhanging_item_choice])
    self.npc.drop(self.npc.items[-1])
    self.hero.send_message(MESSAGE_LOST + self.exhanging_items2[j].dumpsuid())
    
    
class BaseTrap(tofu.Mobile):
  def __init__(self):
    tofu.Mobile.__init__(self)
    
    self.i             = 3.5
    self.j             = 3.5
    self.animation_pos = 0
    
  def loaded(self):
    tofu.Mobile.loaded(self)
    self._init()
    
    
class BaseFountain(BaseTrap, Discutable, Blocker):
  blocking_radius = 0.8
  
  def _init(self):
    self.set_model_name("fontaine")
    self.set_animation_name("attente")
    
  def prepare(self, level, i, j):
    self.i     = i
    self.j     = j
    self.angle = 180.0
    level.add_mobile(self)
    
    self._init()
    
class BaseChest(BaseTrap, Strikeable, Blocker):
  blocking_radius = 0.68
  def __init__(self):
    super(BaseChest, self).__init__()
    
    self.opened = 0
    self.bot    = 1
    
  def prepare(self, level, state_id, i, j, item, open_script = None):
    self.state_id    = state_id
    self.opened      = UNIVERSE_DATA.level_states.get((level.I, level.J, state_id), 0)
    self.i           = i
    self.j           = j
    self.open_script = open_script
    self.item        = item
    level.add_mobile(self)
    
    self._init()
    
  def get_item(self):
    return self.item
  
  def _init(self):
    self.set_model_name("coffre")
    if self.opened: self.set_animation_name("ouvert")
    else:           self.set_animation_name("ferme")
    
  def __getstate__(self):
    return (self.uid, self.player_name, self.local, self.bot, self.level, self.i, self.j, self.state_id, self.opened, self.open_script, self.item)
  
  def __setstate__(self, state):
    self.uid, self.player_name, self.local, self.bot, self.level, self.i, self.j, self.state_id, self.opened, self.open_script, self.item = state
    self.last_noticeable_round = 0
    self.last_important_round  = 0
    
  def hit(self, by):
    if (isinstance(by, BaseHero) or (isinstance(by, BaseShoot) and by.faction == 0)) and (not self.opened):
      self.open()
      if self.open_script:
        if isinstance(by, BaseShoot): by = by.caster
        if isinstance(by, BaseHero): self.open_script(by)
        
  def open(self):
    self.send_message(MESSAGE_OPENED)
    if self.state_id:
      UNIVERSE_DATA.level_states[self.level.I, self.level.J, self.state_id] = 1
      
    j = self.j - 0.6
    item_on_ground = arkanae3.game.ItemOnGround(self.get_item(), self.i, j)
    self.level.add_mobile(item_on_ground)
    
  def do_physics(self):
    if (self.animation_name == "ouvre") and (self.animation_pos == 9):
      self.set_animation_name("ouvert")
      
  def do_message(self, message):
    if message == MESSAGE_OPENED:
      self.opened = 1
      self.set_animation_name("ouvre")
      tofu.MAIN_LOOP.play_sound("mecanism.wav")
      

class BaseGrid(BaseTrap, Blocker):
  blocking_radius = 0.68
  def __init__(self):
    super(BaseGrid, self).__init__()
    
    self.opened = 0
    self.bot    = 1
    
  def prepare(self, level, state_id, i, j, key = u"", open_script = None):
    self.state_id    = state_id
    self.opened      = UNIVERSE_DATA.level_states.get((level.I, level.J, state_id), 0)
    self.i           = i
    self.j           = j
    self.key         = key
    self.open_script = open_script
    level.add_mobile(self)
    
    self._init()
    
  def _init(self):
    self.set_model_name("grille")
    if self.opened: self.set_animation_name("ouvert")
    else:           self.set_animation_name("ferme")
    
  def __getstate__(self):
    return (self.uid, self.player_name, self.local, self.bot, self.level, self.i, self.j, self.state_id, self.opened, self.open_script)
  
  def __setstate__(self, state):
    self.uid, self.player_name, self.local, self.bot, self.level, self.i, self.j, self.state_id, self.opened, self.open_script = state
    self.last_noticeable_round = 0
    self.last_important_round  = 0
    
  def open(self):
    self.send_message(MESSAGE_OPENED)
    if self.state_id:
      UNIVERSE_DATA.level_states[self.level.I, self.level.J, self.state_id] = 1
    
  def close(self):
    self.send_message(MESSAGE_CLOSED)
    if self.state_id:
      UNIVERSE_DATA.level_states[self.level.I, self.level.J, self.state_id] = 0
      
  def do_physics(self):
    if   (self.animation_name == "ouvre") and (self.animation_pos == 9):
      self.set_animation_name("ouvert")
    elif (self.animation_name == "referme") and (self.animation_pos == 9):
      self.set_animation_name("ferme")
      
  def do_message(self, message):
    if   message == MESSAGE_OPENED:
      self.opened = 1
      self.set_animation_name("ouvre")
      tofu.MAIN_LOOP.play_sound("mecanism.wav")
      
    elif message == MESSAGE_CLOSED:
      self.opened = 0
      self.set_animation_name("ouvre")
      tofu.MAIN_LOOP.play_sound("mecanism.wav")
      
  def block(self, character):
    if self.opened: return 1
    if isinstance(character, BaseHero) and self.key:
      key_bunch = character.has_item(BaseKeyBunch)
      if key_bunch and (self.key in key_bunch.keys):
        self.open()
        GridDiscussion(character, self).set_discu("ouvre")
#        return 1
      else:
        GridDiscussion(character, self).set_discu("bloque")
    return 0
        
class GridDiscussion(BaseDiscussion):
  def discu_ouvre(self):
    self.set_text(_(u"Vous ouvrez la grille avec la %s.") % _(self.npc.key).lower())
    
  def discu_bloque(self):
    if self.npc.key:
      self.set_text(_(u"La grille est verrouillée. Vous pourrez l'ouvrir avec la %s.") % _(self.npc.key).lower())
    else:
      self.set_text(_(u"La grille est fermé par un mécanisme."))
    

class BaseMine(BaseTrap, Strikeable):
  def __init__(self):
    super(BaseMine, self).__init__()
    
    self.bot      = 1
    self.disabled = 0
    self.angle    = 0
    self._init()
    
  def __getstate__(self):
    return (self.uid, self.player_name, self.local, self.bot, self.level, self.i, self.j, self.angle, self.disabled)
  
  def __setstate__(self, state):
    self.uid, self.player_name, self.local, self.bot, self.level, self.i, self.j, self.angle, self.disabled = state
    self.last_noticeable_round = 0
    self.last_important_round  = 0
    
  def _init(self):
    self.set_model_name("mine")
    self.set_animation_name("attente")
    
  def do_physics(self):
    if self.disabled: self.disabled -= 1
    
  def do_collisions(self):
    if not self.disabled:
      for character in self.level.mobiles:
        if isinstance(character, BaseCharacter):
          if (character.i - self.i) ** 2 + (character.j - self.j) ** 2 < 0.6:
            character.hit(self)
            self.send_message(MESSAGE_OPENED)
            
  def do_message(self, message):
    if message == MESSAGE_OPENED:
      self.disabled = 40
      
  def advance_time(self, proportion):
    if self.disabled: self.angle = (self.angle + 1.0 * proportion * (self.disabled + 5)) % 360.0
      

class Tool(tofu.Unique):
  key   = None
  price = 0
  def __init__(self):
    tofu.Unique.__init__(self)
    
  def usable_by(self, character): return 1
  
  def description(self): return _(u"__%s__" % self.__class__.__name__)
  
  def calc_stats(self, character): pass
  
  def gained(self, character): pass
  def lost  (self, character): pass
  
  def get_price(self): return self.price
  
class Dropable(object):
  def dropped(self, character, i = None, j = None):
    if isinstance(self, Equipable) and self.equiped: self.set_equiped(character, 0)
    character.items.remove(self)
    
    if self.key:
      character.keys.pop(self.key, None)
      del self.key

    if i is None:
      v = angle2vector(character.angle, 0.4)
      i = character.i + v[0]
      j = character.j + v[1]
      
    if tofu.has_side("server") or tofu.has_side("single"):
      character.level.add_mobile(arkanae3.game.ItemOnGround(self, i, j))
      
class Equipable(object):
  def __init__(self):
    super(Equipable, self).__init__()
    self.equiped = 0
    
  def set_equiped(self, character, equiped):
    if        equiped and not self.equiped: self.on_equiped  (character)
    elif self.equiped and not      equiped: self.on_unequiped(character)
    
  def lost(self, character):
    if self.equiped: character.unequip(self)
    
  def on_equiped  (self, character): self.equiped = 1
  def on_unequiped(self, character): self.equiped = 0
  
class Weapon(Equipable):
  strike_sound = "coup.wav"
  weight       = 1
  def usable_by(self, character):
    if isinstance(character, BaseHero) and (self.weight > character.max_item_weight): return 0
    return super(Weapon, self).usable_by(character)
  
  def on_equiped(self, character):
    for item in character.items:
      if isinstance(item, Weapon) and item.equiped: item.set_equiped(character, 0)
    super(Weapon, self).on_equiped(character)
    character.weapon = self
    character.calc_stats()
    
  def on_unequiped(self, character):
    super(Weapon, self).on_unequiped(character)
    character.weapon = None
    character.calc_stats()
    
  def description(self):
    return "%s (%s +%s, %s)" % (_(u"__%s__" % self.__class__.__name__), _(u"__attack_%s__" % self.damages[1]), self.damages[2], _(u"__attack_range_%s__" % self.damages[0]))
  
class Armor(Equipable):
  weight       = 1
  def usable_by(self, character):
    if isinstance(character, BaseHero) and (self.weight > character.max_item_weight): return 0
    return super(Armor, self).usable_by(character)
  
  def on_equiped(self, character):
    for item in character.items:
      if isinstance(item, Armor) and item.equiped: item.set_equiped(character, 0)
    super(Armor, self).on_equiped(character)
    character.armor = self
    character.calc_stats()
    
  def on_unequiped(self, character):
    super(Armor, self).on_unequiped(character)
    character.armor = None
    character.calc_stats()
    
  def description(self):
    return "%s (%s +%s)" % (_(u"__%s__" % self.__class__.__name__), _(u"protection"), self.protection)
  
class Usable(object):
  use_verb = u"Utiliser"
  def used(self, character): pass
  
class Combinable(object): pass
  

class ChapterDependentPrice(object):
  def get_price(self):
    return super(ChapterDependentPrice, self).get_price() * UNIVERSE_DATA.states.chapitre
  

class Limited(object):
  nb_use = 1
  def used(self, character):
    if isinstance(self, Usable): super(Limited, self).used(character)
    self.nb_use -= 1
    if self.nb_use == 0:
      character.remove_item(self)
      
  def on_equiped(self, character):
    super(Limited, self).on_equiped(character)
    if not isinstance(self, Usable): self.nb_use -= 1 # Else, it is the number of use that is limited (e.g. a weapon)
    
  def on_unequiped(self, character):
    super(Limited, self).on_unequiped(character)
    if self.nb_use == 0:
      character.remove_item(self)
      
  def description(self):
    s = super(Limited, self).description()
    return s + _(u"__item_nb_use__") % self.nb_use
  
  def get_price(self):
    return super(Limited, self).get_price() * self.nb_use
  
  def combine_with(self, item):
    self.nb_use += item.nb_use
    
    
class Item(Tool):
  pass
  
class Power(Tool):
  deprecates = set()
  
class Spell(Usable, Tool):
  min_mana_cost = 0.0
  use_verb      = u"Lancer"
  def used(self, character, target = None):
    if isinstance(character, BaseHero) and (character.mana < self.min_mana_cost):
      character.chat(u"Je n'ai pas assez de mana !")
      return
    character.send_action(ACTION_CAST + self.dumpsuid() + (target or character).dumpsuid())
    
  def casted(self, character, target, mana_cost = None):
    if mana_cost is None: mana_cost = self.min_mana_cost
    character.send_message(MESSAGE_MANA + struct.pack("!f", min(character.mana, character.max_mana) - mana_cost))
    
    
class DodgeTool(Equipable): # Tag class for FLAG_DODGE
  def on_equiped(self, character):
    super(DodgeTool, self).on_equiped(character)
    character.flag |=  FLAG_DODGE
    
  def on_unequiped(self, character):
    super(DodgeTool, self).on_unequiped(character)
    character.flag &= ~FLAG_DODGE

class Paper(Usable, Item):
  use_verb = "Lire"
  
  
class ToolDiscussion(BaseDiscussion):
  def __init__(self, hero, tool):
    BaseDiscussion.__init__(self, hero)
    
    self.tool = tool
    
  def discu_0(self):
    self.set_text(self.tool.description())
    
    if isinstance(self.tool, Equipable):
      if self.tool.equiped: self.add_option(_(u"Déséquiper"), "unequip")
      else:                 self.add_option(_(u"Equiper"  ), "equip"  )
      
    if isinstance(self.tool, Usable):
      self.add_option(_(self.tool.use_verb), "use")
      
    if isinstance(self.tool, Dropable):
      self.add_option(_(u"Jeter"), "drop")
      
    self.add_option(_(u"Ne rien faire"), "cancel")
    
  def discu_equip(self):
    if not self.tool.usable_by(self.hero):
      self.set_text(_(u"__cannotuse__"))
    else:
      self.set_text(u"")
      self.hero.equip(self.tool)
      
  def discu_unequip(self):
    self.set_text(u"")
    self.hero.unequip(self.tool)
    
  def discu_use(self):
    if not self.tool.usable_by(self.hero):
      self.set_text(_(u"__cannotuse__"))
    else:
      self.set_text(u"")
      self.hero.use(self.tool)
      
  def discu_drop(self):
    self.set_text(u"")
    self.hero.drop(self.tool)
    
  def discu_cancel(self):
    self.set_text(u"")
    

class BaseKeyBunch(Dropable, Combinable, Item):
  model_name       = "clef"
  
  def description(self):
    if len(self.keys) == 1:
      return _(tuple(self.keys)[0])
    else:
      return u"%s (%s)" % (_(u"Trousseau"), u", ".join([_(key) for key in self.keys]))
    
  def __init__(self, keys = []):
    Item.__init__(self)
    self.keys = set(keys)
    
  def combine_with(self, item):
    self.keys.update(item.keys)
    
    
class BaseItemOnGround(tofu.Mobile, Discutable):
  def __init__(self, item, i, j):
    tofu.Mobile.__init__(self)
    self.bot  = 1
    self.item = item
    self.i    = i
    self.j    = j
    self.grab_script = None
    
  def __getstate__(self):
    return (self.uid, self.level, self.player_name, self.local, self.bot, self.item, self.i, self.j, self.grab_script)
  
  def __setstate__(self, state):
    self.uid, self.level, self.player_name, self.local, self.bot, self.item, self.i, self.j, self.grab_script = state
    self.last_noticeable_round = 0
    self.last_important_round  = 0
    
  def loaded(self):
    tofu.Mobile.loaded(self)
    if self.item: self.item.loaded()
    
  def discard(self):
    tofu.Mobile.discard(self)
    if self.item: self.item.discard()

  def discute(self,hero):
    ItemOnGroundDiscussion(hero, self).set_discu("0")

    
class ItemOnGroundDiscussion(BaseDiscussion):
  def __init__(self, hero, item_on_ground):
    BaseDiscussion.__init__(self, hero)
    
    self.item_on_ground = item_on_ground
    
  def discu_0(self):
    self.set_text(_(u"Objet au sol : ") + self.item_on_ground.item.description())
    
    self.add_option(_(u"Ramasser"), "grab")
    self.add_option(_(u"Laisser au sol"), "cancel")
    
  def discu_grab(self):
    self.set_text(u"")
    self.hero.grab(self.item_on_ground)
    if self.item_on_ground.grab_script:
      self.item_on_ground.grab_script(self.hero)
      self.item_on_ground.grab_script = None
      
  def discu_cancel(self):
    self.set_text(u"")
    
    
