# -*- coding: utf-8 -*-
# Arkanae III
# Copyright (C) 2008-2009 Jean-Baptiste LAMY
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import os, os.path, time, weakref
import pygame, pygame.image, pygame.color, pygame.mixer, pygame.locals, pygame.joystick

from pygame.locals import *

from arkanae3.base import *
import arkanae3.globdef as globdef

key_code_2_name = dict([(v, k[2:].lower()) for k, v in pygame.locals.__dict__.iteritems() if k.startswith("K_")])

BACKBUFFER          = None
SCREEN_RECT         = None
DIRTY_RECTS         = []
WHITE               = None
BARRE               = None
LIFE                = None
MANA                = None
MISK_MODEL          = None
FONT_MODEL          = None
ITEM_MODEL          = None
ITEM_MODEL_NO_COLOR = None
FOND                = None
FOND_RECT           = None
FOND_RECT2          = None

def init(screen_width = 640, screen_height = 480, fond_gris = "fond_gris.png"):
  global BACKBUFFER, SCREEN_RECT, WHITE, BARRE, LIFE, MANA, MISK_MODEL, FONT_MODEL, ITEM_MODEL, ITEM_MODEL_NO_COLOR, LOG, FOND, FOND_RECT, FOND_RECT2
  
  pygame.init()
  pygame.display.set_caption("Arkanae III")
  SCREEN_RECT = Rect(0, 0, screen_width, screen_height)
  winstyle    = 0
  bestdepth   = pygame.display.mode_ok (SCREEN_RECT.size, winstyle, 32)
  BACKBUFFER  = pygame.display.set_mode(SCREEN_RECT.size, winstyle, bestdepth)
  WHITE       = pygame.color.Color('white')
  
  def load_image(file):
    image = pygame.image.load(os.path.join(globdef.APPDIR, "sprites", file))
    return image.convert()
  
  BARRE       = load_image("barre.png")
  LIFE        = load_image("vie.png")
  MANA        = load_image("mana.png")
  FOND        = load_image(fond_gris)
  
  #t = time.time()
  #get_model("balazar_gourdin")
  #print time.time() - t
  #import sys; sys.exit()
  
  MISK_MODEL          = get_model("divers")
  FONT_MODEL          = FontModel("font")
  ITEM_MODEL          = get_model("item")
  ITEM_MODEL_NO_COLOR = SpriteModel("item")
  ITEM_MODEL_NO_COLOR.colorized = 0
  
  LOG = Log()
  
  FOND_RECT   = Rect(0,  0, LOG.text_width, 20)
  FOND_RECT2  = Rect(0, 20, LOG.text_width,  4)
  
  if pygame.joystick.get_count():
    global JOYSTICK
    JOYSTICK = pygame.joystick.Joystick(0)
    JOYSTICK.init()
    
    
SOUND_DIR = os.path.join(globdef.APPDIR, "sounds")
SOUNDS = {}
class MainLoop(BaseMainLoop):
  def __init__(self):
    global human_controller
    human_controller = HumanController()
    
    BaseMainLoop.__init__(self)
    self.music = None
    #self.play_music("oceane.ogg")
    
  def init_interface(self): pass
  
  def render(self):
    BaseMainLoop.render(self)
    
    for level in self.levels: level.render()
    
    global DIRTY_RECTS
    pygame.display.update(DIRTY_RECTS)
    DIRTY_RECTS = []
    
  def play_sound(self, name):
    sound = SOUNDS.get(name)
    if not sound: sound = SOUNDS[name] = pygame.mixer.Sound(os.path.join(SOUND_DIR, name))
    sound.play()
    
  def play_music(self, name):
    if globdef.ZAURUS: return # No Ogg support yet
    if self.music: self.music.stop()
    self.music = pygame.mixer.Sound(os.path.join(SOUND_DIR, name))
    self.music.play(1)

  def main_loop(self):
    try:
      BaseMainLoop.main_loop(self)
    finally:
      if self.music: self.music.stop()
      human_controller.character = None
      LOG.entries = []
      
      
def j_sorter(a, b):
  if a.life <= 0.0:
    if b.life <= 0.0: return cmp(b.j, a.j)
    return -1
  if b.life <= 0.0: return  1
  return cmp(b.j, a.j)
  
class Level(BaseLevel):
  shot = None
  
  def __init__(self, filename):
    self.effects     = []
    self.renderables = []
    BaseLevel.__init__(self, filename)
    
    self.shot = pygame.image.load(self.room.get_shot_filename()).convert()
    
  def __setstate__(self, state):
    self.effects      = []
    self.renderables  = []
    BaseLevel.__setstate__(self, state)
    self.renderables += self.mobiles
    
    self.shot = pygame.image.load(self.room.get_shot_filename()).convert()
    
  def add_mobile(self, mobile, _send_later = 0):
    BaseLevel.add_mobile(self, mobile, _send_later)
    
    self.renderables.append(mobile)
    
  def remove_mobile(self, mobile):
    BaseLevel.remove_mobile(self, mobile)
    
    self.renderables.remove(mobile)
    
  def set_active(self, active):
    if active and not self.active:
      rect = BACKBUFFER.blit(self.shot, (0, 0))
      if human_controller.character:
        human_controller.character.update_life()
      global DIRTY_RECTS
      DIRTY_RECTS = [rect] # whole screen
      LOG.need_render = 7
      LOG.clear()
      LOG.rect = LOG.input_rect = LOG.item_rect = Rect(0, 0, 0, 0) # No need to clear, the background is fresh!
      
      set_sprite_color(self.room.sprite_color)
      
    BaseLevel.set_active(self, active)
    
  def begin_round(self):
    BaseLevel.begin_round(self)
    #for effect in self.effects[:]: effect.begin_round()
    for effect in self.effects: effect.begin_round()
    LOG.begin_round()
    
  def render(self):
    self.renderables.sort(j_sorter)
    
    for character in self.renderables: character.before_render()
    if LOG.need_render: LOG.before_render(self.shot)
    
    rects = [renderable.dirty_rect for renderable in self.renderables if renderable.can_skip_render <= 1]
    
    if LOG.need_render & 1: rects.append(LOG.rect)
    
    for character in self.renderables: character.render(rects)
    
    if LOG.need_render: LOG.render(self.shot)
    

MODELS = weakref.WeakValueDictionary()
LAST_MODELS = [None] * 3

def get_model(name):
  model = MODELS.get(name)
  if not model:
    model = MODELS[name] = SpriteModel(name)
    LAST_MODELS.append(model)
    del LAST_MODELS[0]
  return model

SPRITE_COLOR = None

def set_sprite_color(color):
  global SPRITE_COLOR
  SPRITE_COLOR = color
  if color:
    for model in MODELS.values():
      if model.colorized: model.colorize(SPRITE_COLOR)
  else:
    for model in MODELS.values():
      if model.colorized: model.uncolorize()
    
    
class SpriteModel(object):
  colorized = 1
  def __init__(self, name):
    t = time.time()
    self.name         = name
    self.image        = pygame.image.load(os.path.join(globdef.APPDIR, "sprites", "%s.png" % name))
    
    if self.colorized:
      self.orig_palette = self.image.get_palette()
      if SPRITE_COLOR: self.colorize(SPRITE_COLOR)
      
    t2 = time.time() - t
    sprite_module = __import__("arkanae3.sprites.%s" % name, fromlist = ["animation_nb", "coords"])
    self.animation_nb = sprite_module.animation_nb
    self.coords       = sprite_module.coords
    t3 = time.time() - t
    print "* Arkanae 3 * Animation %s loaded in %s s (graphics: %s s)." % (name, t3, t2)
    
  def colorize(self, color):
    r0, g0, b0, a0 = color
    a = 1.0 - a0
    palette = [(a * r + a0 * r0, a * g + a0 * g0, a * b + a0 * b0) for (r, g, b) in self.orig_palette]
    self.image.set_palette(palette)
    
  def uncolorize(self):
    self.image.set_palette(self.orig_palette)
    
class FontModel(SpriteModel):
  colorized = 0
  def __init__(self, name):
    SpriteModel.__init__(self, name)
    self._red = (255, 0, 0, 255)
    
  def render_to_image(self, text):
    x = y = width = 0
    for char in text:
      if   char == u" ":
        x += 6
      elif char == u"\n":
        if x > width: width = x
        x  = 0
        y += 20
      else:
        ix, iy, iw, ih, x0, y0 = self.coords[char, 0, 0]
        x += iw
    if x > width: width = x
    height = y + 25
    image = pygame.Surface((width + 1, height))
    image.fill(self._red)
    self.render(text, 0, 0, image)
    image.set_colorkey(self._red, RLEACCEL)
    return image
  
  def render(self, text, x, y, surface):
    start_x = x
    start_y = y
    width   = 0
    for char in text:
      if   char == u" ":
        x += 6
      elif char == u"\n":
        if x - start_x > width: width = x - start_x
        x  = start_x
        y += 20
      else:
        ix, iy, iw, ih, x0, y0 = self.coords[char, 0, 0]
        #surface.blit(self.image, (x + x0, y + y0), (ix, iy, iw, ih))
        surface.blit(self.image, (x, y + y0), (ix, iy, iw, ih))
        x += iw
    if x - start_x > width: width = x - start_x
    return width + 1, y - start_y + 25
  
  def wrap_text(self, text, max_width):
    i   = 0
    x   = 0
    y   = 0
    pos = 0
    last_word = 0
    last_word_width = 0
    new_text = []
    width = 0
    while i < len(text):
      char = text[i]
      if   char == u" ":
        last_word = i
        last_word_width = x
        x += 6
      elif char == u"\n":
        x  = 0
        y += 20
      else:
        ix, iy, iw, ih, x0, y0 = self.coords[char, 0, 0]
        x += iw
        if x > max_width:
          if last_word_width > width: width = last_word_width
          i  = last_word
          x  = 0
          y += 20
          new_text.append(text[pos:i])
          pos = i + 1
      i += 1
    if x > width: width = x
    new_text.append(text[pos:])
    return u"\n".join(new_text), width + 1, y + 25
    
    
class LogEntry(object):
  def __init__(self, text, option_id = None):
    self.orig_text = text
    self.option_id = option_id
    self.duration  = 100 + 2 * len(text)
    if not option_id is None: self.text, self.width, self.height = FONT_MODEL.wrap_text(text, LOG.text_width - 45 - 8) # 45 for " -> "
    else:                     self.text, self.width, self.height = FONT_MODEL.wrap_text(text, LOG.text_width - 8)
    
    self.height -= 5
    
class Log(object):
  nb_item = 3
  def __init__(self):
    self.entries         = []
    self.rect            = Rect(0, 0, 0, 0)
    self.need_render     = 0
    self.max_height      = 25 * 3
    self.input           = u""
    self.input_rect      = Rect(0, 0, 0, 0)
    self.item_rect       = Rect(0, 0, 0, 0)
    self.inventory       = 0
    self.first_item      = 0
    self.selected_item   = 0
    self.in_discussion   = 0
    self.selected_option = 0
    self.text_x          = 64
    self.text_width      = 512
    
  def set_character(self, character):
    self.character = character
    
  def start_input(self):
    self.input = u"- "
    self.need_render |= 2
    
  def add_input(self, s):
    if FONT_MODEL.animation_nb.has_key(s) or (s == u" "):
      self.input += s
      self.need_render |= 2
      
  def backspace_input(self):
    if len(self.input) > 2: self.input = self.input[:-1]
    self.need_render |= 2
    
  def end_input(self, ok = 1):
    if ok and self.input[2:]:
      self.character.send_action(ACTION_SAY + self.input[2:].encode("utf8"))
    self.input = u""
    self.need_render |= 2
    
  def start_inventory(self):
    if LOG.in_discussion: return 0
    if not self.character.items: return 0
    self.inventory = 1
    self.need_render |= 4
    
  def end_inventory(self):
    self.inventory = 0
    self.need_render |= 4
      
  def inventory_activate(self):
    item = self.character.items[self.selected_item]
    self.character.activate_tool(item)
    
  def set_first_item(self, i, relative = 0):
      if relative: self.first_item += i
      else:        self.first_item  = i
      self.need_render   |= 4
      
      if   self.first_item < 0: self.first_item = 0
      elif self.first_item > len(self.character.items) - self.nb_item:
        self.first_item = max(0, len(self.character.items) - self.nb_item)

      if   self.selected_item < self.first_item                   : self.selected_item = self.first_item
      elif self.selected_item > self.first_item + self.nb_item - 1: self.selected_item = self.first_item + self.nb_item - 1
      
  def select_item(self, i, relative = 0):
    if not self.inventory: self.start_inventory()
    else:
      if relative: self.selected_item += i
      else:        self.selected_item  = i
      self.need_render   |= 4
      
    if   self.selected_item < 0                            : self.selected_item = 0
    elif self.selected_item > len(self.character.items) - 1: self.selected_item = len(self.character.items) - 1
    
  def begin_round(self):
    if self.entries and not self.in_discussion:
      self.entries[0].duration -= 1
      if self.entries[0].duration == 0:
        del self.entries[0]
        self.need_render |= 1
        
  def add(self, entry):
    if entry.option_id is None:
      self.in_discussion = 0
      self.entries = [entry]
      
    else:
      self.in_discussion = 1
      self.entries.append(entry)
      
    self.need_render |= 1
    
  def remove(self, entry):
    self.entries.remove(entry)
    self.need_render |= 1
    
  def clear(self):
    self.entries         = []
    self.in_discussion   = 0
    self.selected_option = 0
    self.need_render |= 1

  def save_game(self):
    for mobile in human_controller.character.level.mobiles:
      if isinstance(mobile, BaseMonster) and (mobile.get_relation(human_controller.character) == -1):
        tofu.MAIN_LOOP.play_sound("menu2.wav")
        LOG.clear()
        LOG.add(LogEntry(_(u"Vous ne pouvez pas sauvegarder lorsqu'il y a des ennemis !")))
        return 0
    arkanae3.base.Player.get(human_controller.character.player_name).save()
    return 1
    
  def validate(self):
    if self.in_discussion == 2: # Main menu
      if tofu.has_side("client"):
        if   self.selected_option == 0: LOG.clear() # Continue
        elif self.selected_option == 1: tofu.MAIN_LOOP.stop() # Save and quit
      else:
        if   self.selected_option == 0: LOG.clear() # Continue
        elif self.selected_option == 1: arkanae3.game.start_single.reload = 1; tofu.MAIN_LOOP.stop() # Reload
        elif self.selected_option == 2: LOG.clear(); self.save_game() # Save and continue
        elif self.selected_option == 3:
          if self.save_game(): tofu.MAIN_LOOP.stop() # Save and quit
        elif self.selected_option == 4: tofu.MAIN_LOOP.stop() # Quit
    else:
      self.character.answer(self.selected_option)
      
    if self.inventory: self.end_inventory()
    
    human_controller.up_pressed    = 0
    human_controller.left_pressed  = 0
    human_controller.right_pressed = 0
    human_controller.down_pressed  = 0
    
  def shift_selected_option(self, d):
    self.selected_option = self.selected_option + d
    if   self.selected_option < 0: self.selected_option = self.entries[-1].option_id
    elif self.selected_option > self.entries[-1].option_id: self.selected_option = 0
    self.need_render |= 1
    
  def before_render(self, shot):
    if self.need_render & 1:
      self.old_rect  = BACKBUFFER.blit(shot, self.rect, self.rect)
      
  def render(self, shot):
    if self.need_render & 1:
      total_width  = 0
      total_height = 0
      
      if self.entries:
        for entry in self.entries: total_height += entry.height
        total_height += 5
        
        y = 0
        while y < total_height - 20:
          BACKBUFFER.blit(FOND, (self.text_x, y), FOND_RECT)
          y += 20
        BACKBUFFER.blit(FOND, (self.text_x, y),  Rect(0, 0, self.text_width, total_height - y))
        BACKBUFFER.blit(FOND, (self.text_x, total_height),  FOND_RECT2)
        total_height += 4
        
        y = 0
        for entry in self.entries:
          if entry.option_id == self.selected_option:
            width, height = FONT_MODEL.render(u" --> %s" % entry.text, self.text_x + 7, y, BACKBUFFER)
          elif not entry.option_id is None:
            width, height = FONT_MODEL.render(u"        %s" % entry.text, self.text_x + 7, y, BACKBUFFER)
          else:
            width, height = FONT_MODEL.render(entry.text, self.text_x + 7, y, BACKBUFFER)
          y += entry.height
          
      self.rect = Rect(self.text_x, 0, self.text_width, total_height)
      DIRTY_RECTS.append(self.rect.union(self.old_rect))
      
    if self.need_render & 2:
      old_rect  = BACKBUFFER.blit(shot, self.input_rect, self.input_rect)
      if self.input:
        width, height = FONT_MODEL.render(self.input, 0, 480 - 25, BACKBUFFER)
        self.input_rect = Rect(0, 480 - 25, width, height)
      else:
        self.input_rect = Rect(0, 0, 0, 0)
      DIRTY_RECTS.append(self.input_rect.union(old_rect))
      
    if self.need_render & 4:
      old_rect  = BACKBUFFER.blit(shot, self.item_rect, self.item_rect)
      
      if   self.selected_item < 0                            : self.selected_item = 0
      elif self.selected_item > len(self.character.items) - 1: self.selected_item = max(0, len(self.character.items) - 1)
      if   self.first_item <= self.selected_item - self.nb_item:
        self.first_item = max(0, self.selected_item - self.nb_item + 1)
      elif self.first_item > self.selected_item:
        self.first_item = self.selected_item
      elif self.first_item > len(self.character.items) - self.nb_item:
        self.first_item = max(0, len(self.character.items) - self.nb_item)
        
      if self.first_item > 0:
        ix, iy, iw, ih, x0, y0 = ITEM_MODEL_NO_COLOR.coords[u"^", 0, 0]
        BACKBUFFER.blit(ITEM_MODEL_NO_COLOR.image, (SCREEN_RECT.width - 32 + x0, 25 + y0), (ix, iy, iw, ih))
        
      if self.inventory:
        ix, iy, iw, ih, x0, y0 = ITEM_MODEL_NO_COLOR.coords["cadre", 0, 0]
        BACKBUFFER.blit(ITEM_MODEL_NO_COLOR.image, (SCREEN_RECT.width - 32 + x0, 64 * (self.selected_item - self.first_item) + 32 + 27 + y0), (ix, iy, iw, ih))
        
      y = 27
      for i in range(self.first_item, min(self.first_item + self.nb_item, len(self.character.items))):
        item = self.character.items[i]
        ix, iy, iw, ih, x0, y0 = ITEM_MODEL_NO_COLOR.coords[item.model_name, 0, 0]
        BACKBUFFER.blit(ITEM_MODEL_NO_COLOR.image, (SCREEN_RECT.width - 32 + x0, y + 32 + y0), (ix, iy, iw, ih))
        if isinstance(item, Equipable) and item.equiped:
          ix, iy, iw, ih, x0, y0 = FONT_MODEL.coords[u"x", 0, 0]
          BACKBUFFER.blit(FONT_MODEL.image, (SCREEN_RECT.width + x0 - 18, y + 64 + y0 - 26), (ix, iy, iw, ih))
        if isinstance(item, Limited):
          FONT_MODEL.render(unicode(item.nb_use), SCREEN_RECT.width - 60, y + 64 - 24, BACKBUFFER)
        if item.key:
          FONT_MODEL.render(key_code_2_name[item.key], SCREEN_RECT.width - 60, y, BACKBUFFER)
        y += 64
        
      if self.first_item < len(self.character.items) - self.nb_item:
        ix, iy, iw, ih, x0, y0 = ITEM_MODEL_NO_COLOR.coords[u"v", 0, 0]
        BACKBUFFER.blit(ITEM_MODEL_NO_COLOR.image, (SCREEN_RECT.width - 32 + x0, y + 2 + y0), (ix, iy, iw, ih))
      self.item_rect = Rect(SCREEN_RECT.width - 64, 0, 64, y + 27)
      DIRTY_RECTS.append(self.item_rect.union(old_rect))
    self.need_render = 0
    
LOG = None


class Effect(object):
  life = 1.0 # for j_sorter()
  can_skip_render = 0
  def __init__(self, level, animation_name, i, j, dy = 0, dj = 0.0):
    self.level           = level
    self.animation_name  = animation_name
    self.animation_pos   = 0
    self.i               = i
    self.j               = j + dj
    self.x, self.y       = self.level.room.ij2xy(i, j)
    self.y              += dy
    self.dirty_rect      = self.rect = Rect(self.x, self.y, 0, 0)
    self.nb              = MISK_MODEL.animation_nb[animation_name]
    level.effects    .append(self)
    level.renderables.append(self)
    
  #def begin_round(self):
  #  self.animation_pos += 1
  #  if self.animation_pos == MISK_MODEL.animation_nb[self.animation_name]:
  #    self.level.effects    .remove(self)
  #    self.level.renderables.remove(self)
  #    DIRTY_RECTS.append(BACKBUFFER.blit(self.level.shot, self.rect, self.rect))

  def begin_round(self):
    if self.animation_pos < self.nb: self.animation_pos += 1
      
  def delete(self):
    if self in self.level.effects:
      self.level.effects    .remove(self)
      self.level.renderables.remove(self)
      
  def before_render(self):
    if self.animation_pos < self.nb:
      BACKBUFFER.blit(self.level.shot, self.rect, self.rect)
    else:
      tofu.MAIN_LOOP.next_round_tasks.append(self.delete)
      DIRTY_RECTS.append(BACKBUFFER.blit(self.level.shot, self.rect, self.rect))
      
  def render(self, rects):
    if self.animation_pos < self.nb:
      ix, iy, iw, ih, x0, y0 = MISK_MODEL.coords[self.animation_name, 0, self.animation_pos]
      self.dirty_rect = self.rect
      self.rect = BACKBUFFER.blit(MISK_MODEL.image, (self.x + x0, self.y + y0), (ix, iy, iw, ih))
      self.dirty_rect.union_ip(self.rect)
      DIRTY_RECTS.append(self.dirty_rect)
      
      
class InvisibleMobile(BaseInvisibleMobile):
  can_skip_render = 1
  dirty_rect = Rect(0, 0, 0, 0)
  def before_render(self): pass
  def render(self, rects): pass
  
  
class Animable(object):
  label   = None
  speed_i = 0
  speed_j = 0
  k       = 0
  def _init(self):
    self.can_skip_render = 0
    if self.level: x, y = self.level.room.ij2xy(self.i, self.j)
    else:          x = y = 0
    self.dirty_rect  = self.rect = Rect(x, y, 0, 0)
    
  def change_room(self, special):
    r = super(Animable, self).change_room(special)
    if r:
      x , y = self.level.room.ij2xy(self.i, self.j)
      self.dirty_rect  = self.rect = Rect(x, y, 0, 0)
    return r
  
  def get_model_name(self): return self.model.name
  def set_model_name(self, name):
    self.model = get_model(name)
    self.can_skip_render = 0
    
  def set_animation_name(self, name):
    self.animation_name = name
    self.animation_pos  = 0
    self.animation_nb   = self.model.animation_nb[name]
    self.can_skip_render = 0
    
  def begin_round(self):
    super(Animable, self).begin_round()
    
    self.animation_pos += 1
    if self.animation_pos == self.animation_nb: self.animation_pos = 0
        
  #def render(self):
  #  x , y    = self.level.room.ij2xy(self.i, self.j)
  #  ix, iy, iw, ih, x0, y0 = self.model.coords[self.animation_name, self.animation_angle, self.animation_pos]
  #  self.dirty_rect = self.rect
  #  self.rect = BACKBUFFER.blit(self.model.image, (x + x0, y + y0), (ix, iy, iw, ih))
  #  if self.label:
  #    #self.rect = self.rect.union(BACKBUFFER.blit(self.label, (x - self.label.get_width() // 2, y - 90)))
  #    self.rect.union_ip(BACKBUFFER.blit(self.label, (x - self.label.get_width() // 2, y - 90)))
  #  self.dirty_rect.union_ip(self.rect)
  #  DIRTY_RECTS.append(self.dirty_rect)
    
    
  def before_render(self):
    if self.can_skip_render == 0:
      self.can_skip_render = 1
      BACKBUFFER.blit(self.level.shot, self.rect, self.rect)
    elif (self.animation_nb == 1) and (self.speed_i == 0) and (self.speed_j == 0):
      self.can_skip_render = 2
    else:
      BACKBUFFER.blit(self.level.shot, self.rect, self.rect)
      
  def render(self, rects):
    if self.can_skip_render == 2:
      for rect in rects:
        if self.rect.colliderect(rect): break
      else: return

    x, y = self.level.room.ij2xy(self.i, self.j)
    ix, iy, iw, ih, x0, y0 = self.model.coords[self.animation_name, self.animation_angle, self.animation_pos]
    
    if self.k:
      y -= int(self.k * 48.0)
      self.k -= 0.2
      if self.k < 0.0: self.k = 0.0
      
    if self.can_skip_render == 2:
      BACKBUFFER.blit(self.model.image, (x + x0, y + y0), (ix, iy, iw, ih))
      if self.label:
        BACKBUFFER.blit(self.label, (x - self.label.get_width() // 2, y - 90))
        
    else:
      self.dirty_rect = self.rect
      self.rect = BACKBUFFER.blit(self.model.image, (x + x0, y + y0), (ix, iy, iw, ih))
      if self.label:
        self.rect.union_ip(BACKBUFFER.blit(self.label, (x - self.label.get_width() // 2, y - 90)))
        
      self.dirty_rect.union_ip(self.rect)
      
      DIRTY_RECTS.append(self.dirty_rect)

  def added_into_level(self, level):
    super(Animable, self).added_into_level(level)
    
    if (not level) and self.level:
      # Clean the screen
      DIRTY_RECTS.append(BACKBUFFER.blit(self.level.shot, self.rect, self.rect))
      # Mark all overlapping mobiles so as they don't skip the next rendering
      for mobile in self.level.mobiles:
        if isinstance(mobile, Animable):
          if self.dirty_rect.colliderect(mobile.dirty_rect): mobile.can_skip_render = 0
          
ATTACK_2_EFFECT = { ATTACK_SHARP : "blood", ATTACK_BASH : "blood", ATTACK_LIGHTING : "blood", ATTACK_FIRE : "fire", ATTACK_HEAL : "heal" }



class Character(Animable, BaseCharacter):
  def _init(self):
    BaseCharacter._init(self)
    Animable._init(self)
    self.effect = None
    
  def set_model_name(self, name):
    for weapon in self.items:
      if isinstance(weapon, Weapon) and weapon.equiped:
        try:
          self.model = get_model("%s_%s" % (name, WEAPON_MAP.get(weapon.model_name, weapon.model_name)))
        except:
          self.model = get_model(name)
        break
    else:
      self.model = get_model(name)
      
    self.set_animation_name("attente")
    self.last_anim = None
    
  def set_display_name(self, name):
    BaseCharacter.set_display_name(self, name)
    
    if name and not(isinstance(self, Hero) and tofu.has_side("single")):
      self.label = FONT_MODEL.render_to_image(name)
    else:
      self.label = None
      
  def set_animation_name(self, name):
    Animable.set_animation_name(self, name)
    if   name == "attente": self.animation_angle = (int(round((self.angle) / 45.0)) * 45) % 360
    else:                   self.animation_angle = (int(round((self.angle) / 90.0)) * 90) % 360
    
  def advance_time(self, proportion):
    BaseCharacter.advance_time(self, proportion)
    
    if self.speed_angle:
      if self.animation_name == "attente":
        self.animation_angle = (int(round((self.angle) / 45.0)) * 45) % 360
        self.can_skip_render = 0
      else:
        self.animation_angle = (int(round((self.angle) / 90.0)) * 90) % 360
        self.can_skip_render = 0
        
  def create_effect(self, attack):
    if self.effect and (self.effect in self.level.effects):
      self.effect.delete()
    self.effect = Effect(self.level, ATTACK_2_EFFECT[attack] , self.i, self.j, -30, -0.5)
    
  def do_message(self, message):
    if message[0] == MESSAGE_CHAT:
      #if self.local and not self.bot: 
      #else:                           message = _(u"__chat__") % (self.player_name or self.display_name, message[1:].decode("utf8"))
      if not self.local: return
      message = message[1:].decode("utf8")
      
      if message:
        if LOG.entries and LOG.entries[-1].orig_text == message:
          return
        
        if not LOG.in_discussion: tofu.MAIN_LOOP.play_sound("menu2.wav")
        LOG.add(LogEntry(message))
      else:
        LOG.clear()
        
    elif message.startswith(MESSAGE_ASK):
      if not self.local: return
      message = message[1:].decode("utf8")
      if (not LOG.in_discussion) and message: tofu.MAIN_LOOP.play_sound("menu2.wav")
      
      message, options = message.split(u"\n\n")
      LOG.clear()
      LOG.add(LogEntry(message))
      LOG.entries[-1].height += 10
      i = 0
      for option in options.split(u"\n"):
        LOG.add(LogEntry(option, i))
        i += 1
        
    elif message[0] == MESSAGE_SAY:
      message = _(u"__chat__") % (self.player_name or self.display_name, message[1:].decode("utf8"))
      
      if LOG.entries and LOG.entries[-1].orig_text == message:
        return
      
      if not LOG.in_discussion: tofu.MAIN_LOOP.play_sound("menu2.wav")
      LOG.add(LogEntry(message))
      
      
    else:
      super(Character, self).do_message(message)


class Monster(Character, BaseMonster):
  pass


class Wiz(Animable, BaseWiz):
  animation_angle = 0
  def __init__(self, model_name):
    super(Wiz, self).__init__(model_name)
    
    Animable.set_model_name    (self, "divers")
    Animable.set_animation_name(self, "lightening")
    
  def _init(self):
    BaseWiz._init(self)
    Animable._init(self)
    Animable.set_model_name    (self, "divers")
    Animable.set_animation_name(self, "lightening")
    
  def set_model_name(self, name): pass
  
  def set_animation_name(self, name):
    pass
    #Animable.set_animation_name(self, "lightening")
    
#  def explode(self):
#    self.speed_i = self.speed_j = 0
#    if self.local:
#      self.kill()
  

class Fountain(Animable, BaseFountain):
  animation_angle = 180
  def _init(self):
    BaseFountain._init(self)
    Animable    ._init(self)
    
    if self.level: self.x , self.y = self.level.room.ij2xy(self.i, self.j)
     
  def loaded(self):
    BaseFountain.loaded(self)
    Animable    ._init(self)
    
  def added_into_level(self, level):
    super(Fountain, self).added_into_level(level)
    if self.level: self.x , self.y = self.level.room.ij2xy(self.i, self.j)
    
class Chest(Animable, BaseChest):
  animation_angle = 180
  def _init(self):
    BaseChest._init(self)
    Animable ._init(self)
    if self.level: self.x , self.y = self.level.room.ij2xy(self.i, self.j)
    
  def loaded(self):
    BaseChest.loaded(self)
    Animable ._init(self)
    
  def added_into_level(self, level):
    super(Chest, self).added_into_level(level)
    if self.level: self.x , self.y = self.level.room.ij2xy(self.i, self.j)
    
class Grid(Animable, BaseGrid):
  animation_angle = 180
  def _init(self):
    BaseGrid._init(self)
    Animable ._init(self)
    if self.level: self.x , self.y = self.level.room.ij2xy(self.i, self.j)
    
  def render(self, rects):
    if self.can_skip_render == 2:
      for rect in rects:
        if self.rect.colliderect(rect): break
      else: return
      
    x, y = self.level.room.ij2xy(self.i, 2.75)
    ix, iy, iw, ih, x0, y0 = self.model.coords[self.animation_name, self.animation_angle, self.animation_pos]
    
    if self.k:
      y -= int(self.k * 48.0)
      self.k -= 0.2
      if self.k < 0.0: self.k = 0.0
      
    if self.can_skip_render == 2:
      BACKBUFFER.blit(self.model.image, (x + x0, y + y0), (ix, iy, iw, ih))
      if self.label:
        BACKBUFFER.blit(self.label, (x - self.label.get_width() // 2, y - 90))
        
    else:
      self.dirty_rect = self.rect
      self.rect = BACKBUFFER.blit(self.model.image, (x + x0, y + y0), (ix, iy, iw, ih))
      if self.label:
        self.rect.union_ip(BACKBUFFER.blit(self.label, (x - self.label.get_width() // 2, y - 90)))
        
      self.dirty_rect.union_ip(self.rect)
      
      DIRTY_RECTS.append(self.dirty_rect)

  def loaded(self):
    BaseGrid.loaded(self)
    Animable ._init(self)
    
  def added_into_level(self, level):
    super(Grid, self).added_into_level(level)
    if self.level: self.x , self.y = self.level.room.ij2xy(self.i, self.j)
    
class Mine(Animable, BaseMine):
  life = 1.0
  def _init(self):
    BaseMine._init(self)
    Animable._init(self)
    if self.level: self.x , self.y = self.level.room.ij2xy(self.i, self.j)
    
  def loaded(self):
    BaseMine.loaded(self)
    Animable ._init(self)
    
  def added_into_level(self, level):
    super(Mine, self).added_into_level(level)
    if self.level: self.x , self.y = self.level.room.ij2xy(self.i, self.j)
    
  def set_animation_name(self, name):
    Animable.set_animation_name(self, name)
    
    if name == "attente": self.animation_angle = (int(round((self.angle) / 45.0)) * 45) % 360
    else:                 self.animation_angle = (int(round((self.angle) / 90.0)) * 90) % 360
    
  def advance_time(self, proportion):
    BaseMine.advance_time(self, proportion)
    
    if self.disabled:
      if self.animation_name == "attente":
        self.animation_angle = (int(round((self.angle) / 45.0)) * 45) % 360
        self.can_skip_render = 0
      else:
        self.animation_angle = (int(round((self.angle) / 90.0)) * 90) % 360
        self.can_skip_render = 0
        

    
class ItemOnGround(Animable, BaseItemOnGround):
  life = 1.0
  animation_angle = 0
  animation_nb    = 0
  animation_pos   = 0
  def __init__(self, item, i, j):
    BaseItemOnGround.__init__(self, item, i, j)
    Animable        ._init(self)
    self.set_model_name("item")
    self.animation_name  = self.item.model_name
    
  def loaded(self):
    BaseItemOnGround.loaded(self)
    Animable        ._init(self)
    self.x, self.y = self.level.room.ij2xy(self.i, self.j)
    self.set_model_name("item")
    self.animation_name  = self.item.model_name
    
  def begin_round(self):
    BaseItemOnGround.begin_round(self)
    
  def added_into_level(self, level):
    super(ItemOnGround, self).added_into_level(level)
    
    if level:
      self.x, self.y = self.level.room.ij2xy(self.i, self.j)
      
      

WEAPON_MAP = {
#  "gros_gourdin" : "gourdin",
  }

class Hero(Character, BaseHero):
  def get_model_name(self): return self.model.name.split("_")[0]
        
  def control_owned(self):
    BaseCharacter.control_owned(self)
    
    self.set_controller(human_controller)
    LOG.set_character(self)
    human_controller.current_action = ""
    self.label = None
    
  def control_lost(self):
    BaseCharacter.control_lost(self)
    
    self.set_controller(None)
    
  def update_life(self):
    if self.local:
      DIRTY_RECTS.append(BACKBUFFER.blit(self.level.shot, (0, 0), Rect([0, 0, 64, 64])))
      if self.life > 0.0:
        BACKBUFFER.blit(pygame.transform.scale(HEART, (int(64 * self.life), int(64 * self.life))), (2 + int(32 - 32 * self.life), 2 + int(32 - 32 * self.life)))
        
  def update_life(self):
    if self.local:
      DIRTY_RECTS.append(BACKBUFFER.blit(self.level.shot, (0, 0), Rect([0, 0, 64, 256])))
      BACKBUFFER.blit(BARRE, (0, 0), Rect([0, 0, 64, 32]))
      
      y = 32
      life = min(self.life, self.max_life) # If the player (temporarily) unequip an armor, life > max_life
      for i in range(int(life)):
        BACKBUFFER.blit(LIFE, (0, y), Rect([32, 0, 32, 32])); y += 32
        
      if life != self.max_life:
        l = int(32 * (life % 1.0))
        BACKBUFFER.blit(LIFE, (0, y), Rect([32, 0, 32, l]));
        BACKBUFFER.blit(LIFE, (0, y + l), Rect([ 0, l, 32, 32 - l])); y += 32
        
        for i in range(int(self.max_life) - int(life) - 1):
          BACKBUFFER.blit(LIFE, (0, y), Rect([ 0, 0, 32, 32])); y += 32
          
      y = 32
      mana = min(self.mana, self.max_mana) # cf life
      for i in range(int(mana)):
        BACKBUFFER.blit(MANA, (32, y), Rect([32, 0, 32, 32])); y += 32
        
      if mana != self.max_mana:
        l = int(32 * (mana % 1.0))
        BACKBUFFER.blit(MANA, (32, y), Rect([32, 0, 32, l]));
        BACKBUFFER.blit(MANA, (32, y + l), Rect([ 0, l, 32, 32 - l])); y += 32
        
        for i in range(int(self.max_mana) - int(mana) - 1):
          BACKBUFFER.blit(MANA, (32, y), Rect([ 0, 0, 32, 32])); y += 32
          
          
  def do_message(self, message):
    type = message[0]
    super(Hero, self).do_message(message)
    
    if    type in UPDATE_EQUIP_MSG:
      item = tofu.Unique.undumpsuid(message[1:])
      if isinstance(item, Weapon):
        self.set_model_name(self.get_model_name())
      if self.local: LOG.need_render |= 4
      
    elif (type in UPDATE_MSG) and self.local:
      LOG.need_render |= 4
      
      
UPDATE_EQUIP_MSG = set([MESSAGE_EQUIPED, MESSAGE_UNEQUIPED])
UPDATE_MSG       = set([MESSAGE_USED, MESSAGE_DROPPED, MESSAGE_GRABBED, MESSAGE_GAINED, MESSAGE_LOST])

JOY_BUTTON_KEYS = {
  0 : K_LSHIFT,
  1 : K_RETURN,
  2 : K_ESCAPE,
  3 : K_4,
  4 : K_5,
  5 : K_6,
  6 : K_7,
  7 : K_8,
  }


class HumanController(BaseController):
  def __init__(self):
    BaseController.__init__(self)
    self.current_action = ACTION_STOP_MOVING
    self.character      = None
    self.target         = None
    
    self.up_pressed    = 0
    self.left_pressed  = 0
    self.right_pressed = 0
    self.down_pressed  = 0

  def main_menu(self):
    tofu.MAIN_LOOP.play_sound("menu2.wav")
    LOG.clear()
    LOG.add(LogEntry(_(u"Arkanae3 : Menu principal")))
    LOG.entries[-1].height += 10
    if tofu.has_side("client"):
      LOG.add(LogEntry(_(u"Continuer"), 0))
      LOG.add(LogEntry(_(u"Sauvegarder et quitter"), 1))
    else:
      LOG.add(LogEntry(_(u"Continuer"), 0))
      LOG.add(LogEntry(_(u"Recharger"), 1))
      LOG.add(LogEntry(_(u"Sauvegarder et continuer"), 2))
      LOG.add(LogEntry(_(u"Sauvegarder et quitter"), 3))
      LOG.add(LogEntry(_(u"Quitter sans sauvegarder"), 4))
    LOG.in_discussion = 2
    
  def show_info(self): LOG.add(LogEntry(self.character.get_info()))
  
  def arrow_key_changed(self):
    if   self.left_pressed:
      if   self.up_pressed  : self.character.send_action(ACTION_MOVE_LEFT_UP   )
      elif self.down_pressed: self.character.send_action(ACTION_MOVE_LEFT_DOWN )
      else:                   self.character.send_action(ACTION_MOVE_LEFT      )
    elif self.right_pressed:
      if   self.up_pressed  : self.character.send_action(ACTION_MOVE_RIGHT_UP  )
      elif self.down_pressed: self.character.send_action(ACTION_MOVE_RIGHT_DOWN)
      else:                   self.character.send_action(ACTION_MOVE_RIGHT     )
    elif self.up_pressed:     self.character.send_action(ACTION_MOVE_UP        )
    elif self.down_pressed:   self.character.send_action(ACTION_MOVE_DOWN      )
    else:                     self.character.send_action(ACTION_STOP_MOVING    )
    
  def generate_actions(self):
    events = pygame.event.get()
    
    if not self.character:
      for event in events:
        if (event.type == KEYDOWN) and (event.key == K_ESCAPE): tofu.MAIN_LOOP.stop()
        
    elif LOG.in_discussion:
      for event in events:
        event_type = event.type
        if   event_type == JOYBUTTONDOWN:
          event_type = KEYDOWN
          key        = JOY_BUTTON_KEYS.get(event.button)
        elif event_type == JOYBUTTONUP:
          event_type = KEYUP
          key        = JOY_BUTTON_KEYS.get(event.button)
        else:
          key = getattr(event, "key", None)
          
        if   event_type == KEYDOWN:
          if   (key == K_UP    ) or (key == K_PAGEUP  ): LOG.shift_selected_option(-1)
          elif (key == K_DOWN  ) or (key == K_PAGEDOWN): LOG.shift_selected_option( 1)
          elif (key == K_RETURN) or (key == K_LSHIFT) or (key == K_RSHIFT): LOG.validate()
          
        elif event_type == JOYAXISMOTION:
          if event.axis == 1:
            if   event.value < 0: LOG.shift_selected_option(-1)
            elif event.value > 0: LOG.shift_selected_option( 1)
            
        elif event_type == MOUSEBUTTONDOWN:
          x, y = event.pos
          if   64 < x < SCREEN_RECT.width - 64:
            y2 = 0
            for entry in LOG.entries:
              if (not entry.option_id is None) and (y2 <= y <= y2 + entry.height):
                LOG.selected_option = entry.option_id
                LOG.validate()
                break
              y2 += entry.height
          elif (x >= SCREEN_RECT.width - 64) and (y <= LOG.nb_item * 64 + 64):
            if y < 27: LOG.set_first_item(-1, 1)
            else:
              i = (y - 27) // 64 + LOG.first_item
              if (i >= len(self.character.items)) or (y > 27 + 64 * LOG.nb_item):
                LOG.set_first_item(1, 1)
              else:
                LOG.start_inventory()
                LOG.select_item(i)
                LOG.inventory_activate()
              
            
    else:
      #if self.target:
        #if   (self.character.current_action == ACTION_STRIKE) and ((self.target.life <= 0.0) or ((isinstance(self.target, Chest) and self.target.opened))):
        #  self.character.send_action(ACTION_STOP_MOVING)
        #  self.target = None
        #elif (self.character.current_action == ACTION_STOP_MOVING) and (self.target.life > 0.0) and not isinstance(self.target, Chest):
        #   self.character.send_action(ACTION_STRIKE)
      
      for event in events:
        event_type = event.type
        if   event_type == JOYBUTTONDOWN:
          event_type = KEYDOWN
          key        = JOY_BUTTON_KEYS.get(event.button)
        elif event_type == JOYBUTTONUP:
          event_type = KEYUP
          key        = JOY_BUTTON_KEYS.get(event.button)
        else:
          key = getattr(event, "key", None)
          
        if   event_type == KEYDOWN:
          if   LOG.input:
            if   key == K_ESCAPE   : LOG.end_input(0)
            elif key == K_RETURN   : LOG.end_input(1)
            elif key == K_TAB      : LOG.end_input(1)
            elif key == K_SPACE    : LOG.add_input(u" ")
            elif key == K_BACKSPACE: LOG.backspace_input()
            elif event.unicode     : LOG.add_input(event.unicode)
          else:
            if   key == K_ESCAPE  :
              if LOG.inventory: LOG.end_inventory()
              else:             self.main_menu() #tofu.MAIN_LOOP.stop()
            elif key == K_LEFT    :
              if LOG.inventory: LOG.end_inventory()
              self.left_pressed = 1
              self.arrow_key_changed()
            elif key == K_RIGHT   :
              if LOG.inventory: LOG.end_inventory()
              self.right_pressed = 1
              self.arrow_key_changed()
            elif key == K_UP      :
              if LOG.inventory: LOG.select_item(-1, 1)
              else:
                self.up_pressed = 1
                self.arrow_key_changed()
            elif key == K_DOWN    :
              if LOG.inventory: LOG.select_item( 1, 1)
              else:
                self.down_pressed = 1
                self.arrow_key_changed()
                
            elif (key == K_LSHIFT) or (key == K_RSHIFT):
              if LOG.inventory: LOG.inventory_activate(); LOG.end_inventory()
              else:
                best = None
                for mobile in self.character.level.mobiles:
                  if isinstance(mobile, Discutable):
                    if (abs(mobile.i - self.character.i) < 1.2) and (abs(mobile.j - self.character.j) < 1.2) and mobile.get_relation(self.character) >= 0:
                      if (not best) or (not isinstance(best, ItemOnGround)): best = mobile
                if best:
                  self.character.start_discussion(best)
                else:
                  self.character.send_action(ACTION_STRIKE_ONE)
                  
            elif key == K_i       : self.show_info()
            elif key == K_RETURN  :
              if LOG.inventory: LOG.inventory_activate(); LOG.end_inventory()
              else:             LOG.start_inventory()
            elif key == K_PAGEUP   : LOG.select_item(-1, 1)
            elif key == K_PAGEDOWN : LOG.select_item( 1, 1)
            elif key == K_TAB      : LOG.start_input()
            
            else:
              if LOG.inventory:
                item = LOG.character.items[LOG.selected_item]
                if isinstance(item, Equipable) or isinstance(item, Usable):
                  self.character.set_key(key, item)
                  LOG.need_render |= 4
              else:
                item = self.character.keys.get(key)
                if   isinstance(item, Equipable):
                  if item.equiped: self.character.unequip(item)
                  else:            self.character.equip  (item)
                elif isinstance(item, Usable   ):
                  self.character.use(item)
                  
        elif event_type == KEYUP:
          if not LOG.input:
            if   key == K_LEFT    :
              self.left_pressed = 0
              self.arrow_key_changed()
            elif key == K_RIGHT   :
              self.right_pressed = 0
              self.arrow_key_changed()
            elif key == K_UP      :
              self.up_pressed = 0
              self.arrow_key_changed()
            elif key == K_DOWN    :
              self.down_pressed = 0
              self.arrow_key_changed()
            elif (key == K_LSHIFT) or (key == K_RSHIFT):
              self.character.send_action(ACTION_STOP_MOVING)
              
        elif event_type == MOUSEBUTTONDOWN:
          if event.button == 3: self.character.send_action(ACTION_STRIKE_ONE)
          else:
            x, y = event.pos
            if   (x <= 64 ) and (y < 40 ): self.main_menu()#tofu.MAIN_LOOP.stop()
            elif (x <= 64 ) and (y < 168): self.show_info()
            elif (x >= SCREEN_RECT.width - 64) and (y <= LOG.nb_item * 64 + 64):
              if y < 27: LOG.set_first_item(-1, 1)
              else:
                i = (y - 27) // 64 + LOG.first_item
                if (i >= len(self.character.items)) or (y > 27 + 64 * LOG.nb_item):
                  LOG.set_first_item(1, 1)
                else:
                  LOG.start_inventory()
                  LOG.select_item(i)
                  LOG.inventory_activate()
                  
            else:
              if LOG.inventory: LOG.end_inventory()
              i, j  = self.character.level.room.xy2ij(x, y)
              angle = vector2angle(i - self.character.i, j - self.character.j)
              
              for mobile in self.character.level.mobiles:
                #if (not mobile is self.character) and (-0.5 < mobile.i - i < 0.5) and (-1.0 < mobile.j - j < 0.0):
                if (not mobile is self.character) and ((isinstance(mobile, ItemOnGround) and (-0.5 < mobile.i - i < 0.5) and (-0.5 < mobile.j - j < 0.5)) or ((-0.5 < mobile.i - i < 0.5) and (-1.0 < mobile.j - j < 0.0))):
                  if   isinstance(mobile, Strikeable) and (mobile.life > 0.0) and (mobile.get_relation(self.character) == -1):
                    self.target = mobile
                    self.character.send_action(ACTION_GOTO_STRIKE + struct.pack("!fff", i, j, angle) + mobile.dumpsuid())
                    break
                  elif isinstance(mobile, Discutable):
                    if (abs(mobile.i - self.character.i) < 1.5) and (abs(mobile.j - self.character.j) < 1.5):
                      self.character.start_discussion(mobile)
                      break
                    
              else: self.character.send_action(ACTION_GOTO + struct.pack("!fffH", i, j, angle, 0))
              
        elif event_type == MOUSEBUTTONUP:
          if event.button == 3: self.character.send_action(ACTION_STOP_MOVING)
          
        elif event_type == JOYAXISMOTION:
          if   event.axis == 0:
            if LOG.inventory: LOG.end_inventory()
            if   event.value < 0: self.left_pressed = 1; self.right_pressed = 0
            elif event.value > 0: self.left_pressed = 0; self.right_pressed = 1
            else:                 self.left_pressed = 0; self.right_pressed = 0
          elif event.axis == 1:
            if LOG.inventory:
              if   event.value < 0: LOG.select_item(-1, 1)
              elif event.value > 0: LOG.select_item( 1, 1)
              else:                 self.up_pressed = 0; self.down_pressed = 0
            else:
              if   event.value < 0: self.up_pressed = 1; self.down_pressed = 0
              elif event.value > 0: self.up_pressed = 0; self.down_pressed = 1
              else:                 self.up_pressed = 0; self.down_pressed = 0
          self.arrow_key_changed()
        

human_controller = None

