# -*- coding: utf-8 -*-
# Arkanae III
# Copyright (C) 2008-2009 Jean-Baptiste LAMY
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import sys, os, os.path, struct, random, atexit

import cerealizer
import arkanae3.tofu    as tofu
import arkanae3.globdef as globdef

exec "import arkanae3.driver_%s as driver" % globdef.DRIVER

init = driver.init

class MainLoop    (driver.MainLoop    ): pass
class PlayerID    (driver.PlayerID    ): pass
class Player      (driver.Player      ): pass
class Level       (driver.Level       ): pass
class ItemOnGround(driver.ItemOnGround): pass


class Fountain(driver.Fountain):
  def discute(self, hero):
    hero.chat(_(u"Vous buvez à la fontaine. L'eau est très raffraichissante !"))
    hero.send_message(driver.MESSAGE_LIFE + str(driver.ATTACK_HEAL) + struct.pack("!f", hero.max_life))
    hero.send_message(driver.MESSAGE_MANA                           + struct.pack("!f", hero.max_mana))
    

class Wiz(driver.Wiz): pass


class Chest(driver.Chest): pass

class Grid(driver.Grid): pass

class Mine(driver.Mine):
  damages      = [driver.ATTACK_RANGE_SHORT, driver.ATTACK_FIRE, 5]




class MonsterRain(driver.InvisibleMobile):
  def __init__(self):
    super(MonsterRain, self).__init__()
    self.round = 0
    
  def loaded(self):
    self.round = 0
    
  def do_collisions(self):
    if self.round < 10: self.round += 1
    else:
      self.round = 0
      for hero in self.level.mobiles:
        if isinstance(hero, Ork3): return
        
      for hero in self.level.mobiles:
        if isinstance(hero, driver.Hero):
          if hero.spcs.orage:
            hero.spcs.orage += 1
            if hero.spcs.orage > 16:
              if random.random() < 0.7:
                MonsterRainDiscussion(hero).set_discu("orage")
              else:
                MonsterRainDiscussion(hero).set_discu("eclair")
          else:
            hero.spcs.orage0 += 1
            if random.random() < hero.spcs.orage0 / 70.0 - 0.05:
              MonsterRainDiscussion(hero).set_discu("attention")
              
              
class MonsterRainDiscussion(driver.BaseDiscussion):
  def discu_attention(self):
    self.set_text(_(u"Attention, un gros orage approche... mettez-vous vite à l'abri !"))
    self.hero.spcs.orage = 1
    
  def discu_abris(self):
    if self.hero.spcs.orage:
      self.set_text(_(u"L'orage éclate ; heureusement vous vous êtes abrité à temps !"))
      del self.hero.spcs.orage
      del self.hero.spcs.orage0
      
  def discu_orage(self):
    self.set_text(_(u"L'orage éclate !"))
    self.add_option(_(u"Suite"))
    
  def discu_orage2(self):
    self.set_text(_(u"Il pleut des monstres..."))
    del self.hero.spcs.orage
    del self.hero.spcs.orage0
    
    for i in range(random.randint(1, 3)):
      i = 0.5 + random.random() * 6.0
      j = 0.5 + random.random() * 4.0
      ork = Ork3()
      ork.prepare(self.hero.level, 0, i, j, angle = random.random() * 360.0)
      ork.k = 10.0
      for i in range(30):
        if ork.can_pass(0.0, 0.0): break
        ork.i = 0.5 + random.random() * 6.0
        ork.j = 0.5 + random.random() * 4.0
        
      
  def discu_eclair(self):
    self.set_text(_(u"L'orage éclate !"))
    self.add_option(_(u"Suite"))
    
  def discu_eclair2(self):
    self.set_text(_(u"Attention aux éclairs..."))
    del self.hero.spcs.orage
    del self.hero.spcs.orage0
    
    for i in range(random.randint(3, 6)):
      i = 0.5 + random.random() * 6.0
      j = 0.5 + random.random() * 4.0
      angle = random.random() * 360.0
      speed_i, speed_j = driver.angle2vector(angle, 0.12)
      eclair = Wiz("wiz")
      eclair.prepare(self.hero.level, i, j, speed_i, speed_j, [0.5, driver.ATTACK_LIGHTING, 12.0], 4, [-1, -1, -1, -1, -1])
      for i in range(30):
        if eclair.can_pass(0.0, 0.0): break
        eclair.i = 0.5 + random.random() * 6.0
        eclair.j = 0.5 + random.random() * 4.0
        



class KeyBunch(driver.BaseKeyBunch):
  pass

class Robe(driver.Armor, driver.Dropable, driver.Item):
  model_name       = "robe"
  protection       = 0
  resistance       = [0.0, -0.1, -0.05, 0.1]
  price            = 20
  
class RobeDeVerre(driver.Armor, driver.Dropable, driver.Item):
  model_name       = "robe_verre"
  protection       = -1
  resistance       = [0.0, -0.1, -0.05, 0.1]
  price            = 20
  #def set_equiped(self, character, equiped):
  #  super(RobeDeVerre, self).set_equiped(character, equiped)
  
  def description(self): return _(u"Robe de verre (coeur -1, mana +1)")
  def calc_stats(self, character):
    if isinstance(character, Hero) and self.equiped:
      character.max_mana += 1.0
      
    
class Plastron(driver.Armor, driver.Dropable, driver.Item):
  model_name       = "plastron"
  protection       = 1
  resistance       = [0.0, -0.3, -0.1, 0.1]
  price            = 50
  
class CotteDeMaille(driver.Armor, driver.Dropable, driver.Item):
  model_name       = "cottedemaille"
  protection       = 2
  resistance       = [0.0, -0.4, 0.2, 0.0]
  weight           = 2
  price            = 120
  
class Plate(driver.Armor, driver.Dropable, driver.Item):
  model_name       = "plate"
  protection       = 3
  resistance       = [0.0, -0.5, 0.2, 0.0]
  weight           = 3
  price            = 250

  
class Club(driver.Weapon, driver.Dropable, driver.Item):
  damages          = [driver.ATTACK_RANGE_SHORT, driver.ATTACK_BASH, 1]
  model_name       = "gourdin"
  strike_animation = "frappe"
  price            = 10
  
class Knife(driver.Weapon, driver.Dropable, driver.Item):
  damages          = [driver.ATTACK_RANGE_SHORT, driver.ATTACK_SHARP, 1]
  model_name       = "couteau"
  strike_sound     = "strike2.wav"
  strike_animation = "frappe_t"
  price            = 30
  
class Staff(driver.Weapon, driver.Dropable, driver.Item):
  damages          = [driver.ATTACK_RANGE_LONG, driver.ATTACK_BASH, 0]
  model_name       = "baton"
  strike_animation = "frappe_a"
  price            = 30
  

class Axe(driver.Weapon, driver.Dropable, driver.Item):
  damages          = [driver.ATTACK_RANGE_SHORT, driver.ATTACK_BASH, 2]
  model_name       = "hache"
  strike_animation = "frappe"
  weight           = 2
  price            = 70
  
class Sword(driver.Weapon, driver.Dropable, driver.Item):
  damages          = [driver.ATTACK_RANGE_SHORT, driver.ATTACK_SHARP, 3]
  model_name       = "epee"
  strike_sound     = "strike2.wav"
  strike_animation = "frappe_t"
  weight           = 2
  price            = 100

class Lance(driver.Weapon, driver.Dropable, driver.Item):
  damages          = [driver.ATTACK_RANGE_LONG, driver.ATTACK_SHARP, 1]
  model_name       = "lance"
  strike_animation = "frappe_a"
  weight           = 2
  price            = 90
  
  
  
class FireWand(driver.Limited, driver.Weapon, driver.Usable, driver.Dropable, driver.Item):
  damages      = [driver.ATTACK_RANGE_SHORT, driver.ATTACK_FIRE, 3]
  model_name   = "baguette_feu"
  nb_use       = 33
  strike_sound = "fire1.wav"
  
class FireStaff(driver.Limited, driver.Weapon, driver.Usable, driver.Dropable, driver.Item):
  damages      = [driver.ATTACK_RANGE_LONG, driver.ATTACK_FIRE, 2]
  model_name   = "baton_feu"
  nb_use       = 33
  strike_sound = "fire1.wav"

  
class LifePotion(driver.Limited, driver.Usable, driver.Dropable, driver.Combinable, driver.ChapterDependentPrice, driver.Item):
  model_name = "potion_de_vie"
  price      = 15
  use_verb   = u"Boire"
  @tofu.side("server", "single")
  def used(self, character):
    super(LifePotion, self).used(character)
    character.send_message(driver.MESSAGE_LIFE + str(driver.ATTACK_HEAL) + struct.pack("!f", character.max_life))
    
class ManaPotion(driver.Limited, driver.Usable, driver.Dropable, driver.Combinable, driver.ChapterDependentPrice, driver.Item):
  model_name = "potion_de_mana"
  price      = 20
  use_verb   = u"Boire"
  @tofu.side("server", "single")
  def used(self, character):
    super(ManaPotion, self).used(character)
    character.send_message(driver.MESSAGE_MANA + struct.pack("!f", character.max_mana))
    
class MaxPotion(driver.Limited, driver.Usable, driver.Dropable, driver.Combinable, driver.Item):
  model_name = "potion_dexperience"
  price      = 200
  use_verb   = u"Boire"
  def used(self, character):
    super(MaxPotion, self).used(character)
    character.intelligence += 1
    character.calc_stats()
    
class TeleportationPotion(driver.Limited, driver.Usable, driver.Dropable, driver.Combinable, driver.Item):
  model_name = "potion_de_tel"
  price      = 0
  use_verb   = u"Boire"
  @tofu.side("server", "single")
  def used(self, character):
    super(TeleportationPotion, self).used(character)
    while 1:
      character.i = random.uniform(0.3, 6.7)
      character.j = random.uniform(0.3, 4.7)
      if character.can_pass(0.0, 0.0): break
    character.set_current_state_importance(2)
    character.set_animation_name("attente") # Hack ; needed to disable 2D driver "skip render" optimization
    if character.spcs.sriss_potion_tel:
      del character.spcs.sriss_potion_tel
    
  
class HealingSpell(driver.Spell, driver.Power):
  model_name = "sort_soin"
  min_mana_cost = 0.000001
  def casted(self, character, target):
    if target.life < target.max_life:
      power = 2.0 * (target.max_life - target.life) / target.max_life
      if power > character.mana: power = character.mana
      target   .send_message(driver.MESSAGE_LIFE + str(driver.ATTACK_HEAL) + struct.pack("!f", target.life + power * target.max_life * 0.5))
      character.send_message(driver.MESSAGE_MANA + struct.pack("!f", character.mana - power))
      
      
class LighteningWiz(driver.Spell, driver.Power):
  model_name = "sort_wiz"
  min_mana_cost = 0.2
  def casted(self, character, target = None):
    driver.Spell.casted(self, character, target)
    character.send_message(driver.MESSAGE_SOUND + "fire1.wav")
    shoot = Wiz("wiz")
    shoot.caster = character
    speed_i0, speed_j0 = driver.angle2vector(character.angle + 12.0, 0.12)
    speed_i , speed_j  = driver.angle2vector(character.angle       , 0.12)
    shoot.prepare(character.level, character.i + 7.0 * speed_i0, character.j + 7.0 * speed_j0, speed_i, speed_j, [0.5, driver.ATTACK_LIGHTING, 15.0], character.faction, character.faction_relations)
    



class PaperDiscussion(driver.BaseDiscussion):
  def discu_sriss(self):
    self.set_text(_(u"Ces papiers contiennent les plans d'une attaque des Fides contre la cathédrale d'Akéron : contourner les défenses des échassiens par l'ouest."))
    self.hero.spcs.miniquete_nego_papier_sriss = 1
    self.add_option()
    
  def discu_sriss2(self):
    u"Suite"
    self.set_text(_(u"Ces plans ont dû être bouleversés par la catastrophe, car aucune armée ne pourra franchir les terres ravagées de l'ouest avant longtemps."))
    self.add_option()
    
  def discu_sriss3(self):
    u"Suite"
    self.set_text(u"")
      
class SrissPaper(driver.Paper):
  model_name = "papier"
  def used(self, character):
    PaperDiscussion.to_script("sriss")(character)
    
  
  
class EchassianDisguise(driver.Limited, driver.Equipable, driver.Power):
  model_name = "deguisement_echassien"
  nb_use     = 10
  def on_equiped  (self, character):
    super(EchassianDisguise, self).on_equiped  (character)
    character.set_model_name("echassien")
    character.set_display_name(u"")
    character.set_animation_name("attente")
  def on_unequiped(self, character):
    super(EchassianDisguise, self).on_unequiped(character)
    character.set_model_name(character.default_model_name)
    character.set_display_name(character.player_name)
    character.set_animation_name("attente")
    
    
class Hero(driver.Hero):
  model_name = "vagabon"
  #model_name = "mage"
  #model_name = "traite"
  #model_name = "guerrier"
  #model_name = "pretre"
  #model_name = "chevalier"
  #model_name = "voleur"
  #model_name = "sorcier"
  #model_name = "barbare"
  #model_name = "assassin"
  #model_name = "gardien"
  #model_name = "comedien"
  #model_name = "invocateur"
  move_speed = 0.08
  
  resistances = [1.0, 1.3, 1.0, 1.0]
  

class Ork(driver.Monster):
  faction           = driver.FACTION_BERSERK
  faction_relations = [-1, -1, -1, -1, 1]
  move_speed        = 0.09
  strength          = 1
  endurance         = 1
  resistances       = [1.0, 1.2, 1.0, 1.0]
  hit_sound         = die_sound = "couic_fide.wav"
  strike_sound      = "coup.wav"
  
  def __init__(self):
    driver.Monster.__init__(self)
    self.model_name = "orc"


class Ork2(Ork):
  strength          = 2
  endurance         = 2
  
  def __init__(self):
    Ork.__init__(self)
    self.model_name = "orc2"


class Ork3(Ork):
  strength          = 2
  endurance         = 2
  resistances       = [1.0, 1.2, 0.7, 1.0]
  
  def __init__(self):
    Ork.__init__(self)
    self.model_name = "orc3"





class Vagabon(driver.Monster):
  faction           = driver.FACTION_HERO
  faction_relations = [1, 0, 0, 0, -1]
  move_speed        = 0.09
  strength          = 2
  endurance         = 3
  resistances = [1.0, 1.3, 1.0, 1.0]
  hit_sound    = die_sound = "couic1.wav"
  
  def __init__(self):
    driver.Monster.__init__(self)
    self.model_name = "vagabon"
    

class Echassien(driver.Monster):
  faction           = driver.FACTION_ECHASSIAN
  faction_relations = [1, 1, 0, -1, -1]
  resistances       = [1.0, 1.3, 0.9, 1.1]
  hit_sound         = die_sound = "couic_echassien.wav"
  
class ChevalierEchassien(Echassien):
  move_speed        = 0.09
  strength          = 2
  endurance         = 3
  
  def __init__(self):
    Echassien.__init__(self)
    self.model_name = "chevalier"
    self.flag |= driver.FLAG_LARGE_SIZE
    
class PretreEchassien(Echassien):
  def __init__(self):
    Echassien.__init__(self)
    self.model_name = "pretre"
    
class VoleurEchassien(Echassien):
  def __init__(self):
    Echassien.__init__(self)
    self.model_name = "voleur"
    

class Lige(driver.Monster):
  faction           = driver.FACTION_LIGE
  faction_relations = [1, 0, 1, 0, -1]
  resistances       = [1.0, 1.3, 1.0, 1.0]
  hit_sound         = die_sound = "couic_lige.wav"
  
class InvocateurLige(Lige):
  def __init__(self):
    Lige.__init__(self)
    self.model_name = "invocateur"
    

class Fide(driver.Monster):
  faction           = driver.FACTION_FIDE
  faction_relations = [1, 0, 1, 0, -1]
  resistances       = [1.0, 1.3, 1.15, 0.95]
  hit_sound         = die_sound = "couic_fide.wav"
  
class BarbareFide(Fide):
  move_speed        = 0.09
  strength          = 2
  endurance         = 3
  
  def __init__(self):
    Fide.__init__(self)
    self.model_name = "barbare"
    self.flag |= driver.FLAG_RAGE
    
    
class BarbareFide0(BarbareFide): # Les gardes de Sriss
  pass

class BarbareFide1(BarbareFide): # Les trois barbares
  pass
  

class SorcierFide(Fide):
  move_speed        = 0.09
  strength          = 2
  endurance         = 3
  
  def __init__(self):
    driver.Monster.__init__(self)
    self.model_name = "sorcier"



cerealizer.register(Player, tofu.SavedInAPathHandler(Player))
cerealizer.register(Level , tofu.SavedInAPathHandler(Level ))
cerealizer.register(Ork)
cerealizer.register(Ork2)
cerealizer.register(Ork3)
cerealizer.register(Vagabon)
cerealizer.register(InvocateurLige)
cerealizer.register(BarbareFide)
cerealizer.register(BarbareFide0)
cerealizer.register(BarbareFide1)
cerealizer.register(SorcierFide)
cerealizer.register(ChevalierEchassien)
cerealizer.register(PretreEchassien)
cerealizer.register(VoleurEchassien)
cerealizer.register(Hero)
cerealizer.register(Wiz)
cerealizer.register(Fountain)
cerealizer.register(Chest)
cerealizer.register(Grid)
cerealizer.register(Mine)
cerealizer.register(MonsterRain)
cerealizer.register(ItemOnGround)
cerealizer.register(Club)
cerealizer.register(Staff)
cerealizer.register(Axe)
cerealizer.register(Knife)
cerealizer.register(Sword)
cerealizer.register(Lance)
cerealizer.register(FireWand)
cerealizer.register(FireStaff)
cerealizer.register(LifePotion)
cerealizer.register(MaxPotion)
cerealizer.register(ManaPotion)
cerealizer.register(TeleportationPotion)
cerealizer.register(HealingSpell)
cerealizer.register(LighteningWiz)
cerealizer.register(KeyBunch)
cerealizer.register(Robe)
cerealizer.register(RobeDeVerre)
cerealizer.register(Plastron)
cerealizer.register(CotteDeMaille)
cerealizer.register(Plate)
cerealizer.register(SrissPaper)
cerealizer.register(EchassianDisguise)

tofu.LOAD_PLAYER_ID = PlayerID.loads
tofu.CREATE_PLAYER  = Player


  
def start_single():
  start_single.reload = 0
  tofu.set_side("single")
  tofu.SAVED_GAME_DIR = globdef.SAVED_GAME_DIR
  tofu.GAME           = globdef.SINGLE_LOGIN
  tofu.PLAYER_IDS     = [PlayerID(globdef.SINGLE_LOGIN, globdef.SINGLE_LOGIN)]
  main_loop = MainLoop()
  if not globdef.LIMIT_FPS: main_loop.min_frame_duration = 0.0
  main_loop.main_loop()
  print getattr(main_loop, "fps", "-"), "FPS"

  if start_single.reload:
    start_single()

def start_multi():
  if SERVER_PID: # Connect to local server
    tofu.HOST = globdef.SERVER_HOST
    tofu.PORT = globdef.SERVER_PORT
  else:
    tofu.HOST = globdef.CLIENT_HOST
    tofu.PORT = globdef.CLIENT_PORT
    
  tofu.set_side("client")
  tofu.SAVED_GAME_DIR = globdef.SAVED_GAME_DIR
  tofu.GAME           = globdef.CLIENT_LOGIN
  tofu.PLAYER_IDS     = [PlayerID(globdef.CLIENT_LOGIN, globdef.CLIENT_PASSWORD)]
  main_loop = MainLoop()
  if not globdef.LIMIT_FPS: main_loop.min_frame_duration = 0.0
  main_loop.main_loop()
  print getattr(main_loop, "fps", "-"), "FPS"
  
SERVER_PID = 0

def start_server_spawn():
  global SERVER_PID
  SERVER_PID = os.spawnl(os.P_NOWAIT, sys.executable, sys.executable, sys.argv[0], "--server", globdef.SERVER_HOST, "--port", str(globdef.SERVER_PORT))
  
def start_server():
  tofu.HOST = globdef.SERVER_HOST
  tofu.PORT = globdef.SERVER_PORT
  tofu.GAME = "_server"
  
  tofu.set_side("server")
  tofu.SAVED_GAME_DIR = globdef.SAVED_GAME_DIR
  
  universe = driver.UniverseData.load()
  
  main_loop = MainLoop()
  main_loop.main_loop()
  
  universe.save()
  
def kill_server():
  global SERVER_PID
  if SERVER_PID: 
    import signal
    os.kill(SERVER_PID, signal.SIGINT)
    SERVER_PID = 0
atexit.register(kill_server)

def erase_server_data():
  server_running = SERVER_PID
  if server_running: kill_server()
  
  import shutil
  try:    shutil.rmtree(os.path.join(globdef.SAVED_GAME_DIR, "_server"))
  except: pass
  
  if server_running: start_server_spawn()
  
