# -*- coding: utf-8 -*-
# Arkanae III
# Copyright (C) 2008-2009 Jean-Baptiste LAMY
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from arkanae3.rooms import Room

room = Room([
  6,  6,  6,  6,  6,  6,  6,  2,  6,
  6,  6,  6,  7,  7,  0,  0,  0,  6,
  6,  0,  0,  0,  0,  0,  0,  0,  6,
  6,  6,  0,  0,  0,  0,  0,  0,  6,
  6,  0,  0,  0,  0,  0,  0,  0,  6,
  6,  0,  0,  0,  0,  0,  6,  6,  6,
  6,  6,  6,  6,  6,  6,  6,  6,  4,
  ],
  )

room.visible_rooms = [(-1, 1), (0, 1), (1, 1), (-1, 0), (0, 0), (1, 0), (0, -1)]

#room.y_offset = -4.0
room.atmosphere = 2

def build(room_builder):
  room_builder.build_terrain(["grass7_1", "grass7_2", "grass7_3", "grass7_1", "grass7_2", "grass7_3", "grass7_1", "grass7_2", "grass7_3", "grass7_1", "grass7_2", "grass7_3", "grass7_4"], "grass_t1_t2", "grass_t2_t2", "sol_bis", "grass4_t1", "grass4_t2", "grass4")
  #room_builder.build_floor("pentacle3")
  #room_builder.build_wall ()
  
  import soya
  b = soya.Body(room_builder.room_world, soya.Model.get("table"))
  b.set_xyz(-0.3, 0.0, -2.0)
  b = soya.Body(room_builder.room_world, soya.Model.get("tabouret"))
  b.set_xyz(-1.1, 0.0, -2.0)
  #b.rotate_lateral(-90.0)
  
def prepare(level):
  import arkanae3.game
  
  if arkanae3.base.UNIVERSE_DATA.states.sriss_dans_tente:
    arkanae3.game.SorcierFide().prepare(level, 3, 2.5, 1.0, items = [arkanae3.game.Knife()], discute_script = SrissDiscussion.to_script("0"), faction = 0)
    
  else:
    arkanae3.game.Chest().prepare(level, 1, 0.5, 1.5, arkanae3.game.SrissPaper())
    arkanae3.game.BarbareFide0().prepare(level, 2, 3.5, 2.5, items = [arkanae3.game.Sword(), arkanae3.game.Plastron()], faction = 4)


import arkanae3.base, arkanae3.game

class SrissDiscussion(arkanae3.base.BaseDiscussion):
  def add_options(self):
    if self.hero.spcs.chapitre <= 2:
      self.add_option(discu = "sceptre")
      self.add_option(discu = "oui")
      self.add_option(discu = "non")
    else:
      self.add_option(discu = "sceptre")
      if not self.hero.spcs.sriss_potion_tel:
        self.add_option(discu = "potion")
    self.add_option(discu = "fin")
    
  def discu_0(self):
    self.hero.spcs.sait_sriss_veut_sceptre_akeron = 1
    if self.hero.spcs.chapitre <= 2:
      self.set_text(_(u"""Sriss : Bonjour, sss. Je souhaite récupérer le sceptre d'Akéron, qui se trouve actuellement au palais des Liges, et j'aurais besoin de louer les services d'un aventurier compétent pour cela. Je pensssais naturellement à vous... Qu'en dites-vous ?"""))
    else:
      self.set_text(_(u"""Sriss : Bonjour. Je vous ai demandé de récupérer le sceptre d'Akéron, au palais des Liges. Que puis-je pour vous ?"""))
      
    self.add_options()
    
  def discu_sceptre(self):
    u"""Quel est ce sceptre ?"""
    self.set_text(_(u"""Sriss : C'est une babiole, rien de plusss. Les Fides ne révèrent pas Akéron, cependant mon intérêt pour cet objet est purement esthétique. Les Liges se le sont approprié, c'est pourquoi j'aurais besoin d'un voleur... sss..."""))
    self.add_options()
    
  def discu_non(self):
    u"""Je ne suis pas intéressé."""
    self.set_text(_(u"""Sriss : Réfléchissez bien, je vous donne l'occasion de quitter l'ouest ravagé et de découvrir le reste du monde d'Arkanae... sss... revenez me voir quand vous aurez changé d'avis."""))
    
  def discu_oui(self):
    u"""Je suis intéressé."""
    self.set_text(_(u"""Sriss : Bien, bien ! Je savais que vous accepteriez. Vous serez richement récompensé, sss..."""))
    self.add_option()
    
  def discu_oui2(self):
    u"""Suite"""
    self.set_text(_(u"""    Arkanae III - Chapitre 3 : l'aventurier"""))
    self.hero.spcs.chapitre = 3
    if arkanae3.base.UNIVERSE_DATA.states.chapitre < 3:
      arkanae3.base.UNIVERSE_DATA.states.chapitre = 3
    self.add_option()
    
    #self.set_text(_(u"""Sriss : Dorénavant, méfiez-vous des Liges, ces fouineurs ont des oreilles partout et ils feront tout pour vous arrêter."""))
    
  def discu_oui3(self):
    u"""Suite"""
    self.set_text(_(u"""Sriss : Pour atteindre le palais des Liges, traversez la route de l'est à l'aide de cette potion. Si vous ratez votre téléportation, revenez me voir et je vous en donnerai une autre."""))
    self.npc.add_item(arkanae3.game.TeleportationPotion())
    self.npc.drop(self.npc.items[-1])
    self.hero.spcs.sriss_potion_tel = 1
    self.add_options()
    
  def discu_potion(self):
    u"""J'ai raté la téléportation..."""
    self.set_text(_(u"""Sriss : Incapable ! Crétin ! Je vais vous..."""))
    self.add_option()
    
  def discu_potion2(self):
    u"""Suite"""
    self.set_text(_(u"""Sriss : Excusez-moi, je m'emporte... Voici une nouvelle potion. Pour atteindre le palais des Liges, allez le plus loin possible sur la route de l'est, puis buvez cette potion de téléportation."""))
    self.npc.add_item(arkanae3.game.TeleportationPotion())
    self.npc.drop(self.npc.items[-1])
    self.hero.spcs.sriss_potion_tel = 1
    self.add_options()
    
  def discu_fin(self):
    u"Au revoir."
    self.set_text(u"")
