# -*- coding: utf-8 -*-
# Arkanae III
# Copyright (C) 2008-2009 Jean-Baptiste LAMY
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from arkanae3.rooms import Room

room = Room([
  4,  4,  4,  4,  4,  4,  4,  4,  4,
  4,  4,  4,  4,  4,  4,  4,  0,  6,
  6,  0,  0,  0,  4,  4,  0,  0,  6,
  6,  0,  0,  0,  4,  4,  0,  0,  6,
  6,  0,  0,  0,  0,  0,  0,  0,  2,
  6,  0,  0,  0,  4,  4,  0,  0,  2,
  6,  2,  2,  4,  4,  4,  4,  4,  4,
  ],
  )

room.visible_rooms = [(-1, 1), (0, 1), (1, 1), (-1, 0), (0, 0), (1, 0), (0, -1)]

def build(room_builder):
  room_builder.build_water("eau_bleu")
  room_builder.set_limits(x2 = 0.5)
  room_builder.build_terrain(["sol", "sol_bis", "sol_bis"], "rock_t1_t2", "rock_t2_t2", "sol_bis", "rock_t1", "rock_t2", "rock")

  room_builder.set_limits(x1 = 0.5)
  room_builder.build_terrain(["grass7_1", "grass7_2", "grass7_3", "grass7_1", "grass7_2", "grass7_3", "grass7_1", "grass7_2", "grass7_3", "grass7_1", "grass7_2", "grass7_3", "grass7_4"], "grass_t1_t2", "grass_t2_t2", "sol_bis", "grass4_t1", "grass4_t2", "grass4")
  
def prepare(level):
  import arkanae3.game
  arkanae3.game.Chest().prepare(level, 1, 1.5, 3.5, arkanae3.game.HealingSpell())
  arkanae3.game.ChevalierEchassien().prepare(level, 2, 4.0, 1.5, items = [arkanae3.game.Staff(), arkanae3.game.Plastron()], faction = 0, discute_script = Discussion.to_script("0"), die_script = Discussion.to_script("couic"), angle = 180.0, bonus = arkanae3.game.Staff(), life = 1.5)



import arkanae3.base, arkanae3.game
class Discussion(arkanae3.base.BaseDiscussion):
  def add_options(self):
    self.add_option(discu = "tete")
    self.add_option(discu = "aide")
    self.add_option(discu = "fin")
    self.add_option(discu = "combat")
    
  def discu_couic(self):
    arkanae3.base.UNIVERSE_DATA.states.ech_couic = 1
    
  def discu_0(self):
    if self.npc.life > 3.0:
      self.set_text(_(u"""Ech : Encore merci ! Nous nous reverrons peut-être au camp des réfugiés !"""))
    else:
      self.set_text(_(u"""Ech : Ne vous approchez pas ! J'ai attrapé la peste lors de la catastrophe !"""))
      self.add_options()
      
  def discu_tete(self):
    u"Pourquoi avez-vous une tête d'oiseau ?"
    self.set_text(_(u"""Ech : Mais au nord, tous les échassiens ont une tête d'oiseau !"""))
    self.add_options()
    
  def discu_aide(self):
    u"Puis-je vous aider ?"
    
    #self.set_text(_(u"""Ech : Peut-être... j'ai un sort de soin dans mon coffre, mais je n'ai plus de mana pour l'utiliser. Prenez-le !"""))
    self.set_text(_(u"""Ech : Non... j'ai un sort de soin dans mon coffre, mais il faudrait de la mana pour l'utiliser. Tout ce que vous pouvez faire, c'est abréger mes souffrances !"""))
    if self.hero.has_item(arkanae3.game.HealingSpell):
      self.add_option()
    self.add_options()
    
  def discu_aide2(self):
    u"(Lancer un sort de soin)"
    if self.hero.mana < 0.9:
      self.set_text(_(u"""Ech : Vous non plus vous n'avez pas assez de mana. Tout est foutu, Akéron m'a abandonné..."""))
      self.add_options()
    else:
      arkanae3.base.UNIVERSE_DATA.states.ech_gueri = 1
      self.hero.has_item(arkanae3.game.HealingSpell).used(self.hero, self.npc)
      self.set_text(_(u"""Ech : Ah, ça fait du bien ! Merci ! Je vais pouvoir retourner au camp des réfugiés. Gardez mon sort de soin en remerciement !"""))
      self.add_option()
      
  def discu_aide3(self):
    u"Suite"
    self.set_text(u"")
    self.npc.goto(7.0, 1.5)
    arkanae3.base.UNIVERSE_DATA.level_states[self.npc.level.I, self.npc.level.J, self.npc.state_id] = 1
      
  def discu_combat(self):
    u"Je vais abréger vos souffrances !"
    self.set_text(_(u"""Ech : Je dois mourir en chevalier, aussi veuillez m'excuser si je me défends. """))
    self.add_option()
    
  def discu_combat2(self):
    u"Suite"
    self.set_text(u"")
    self.npc.faction = 4
    
  def discu_fin(self):
    u"Au revoir."
    self.set_text(u"")
