# -*- coding: utf-8 -*-
# Arkanae III
# Copyright (C) 2008-2009 Jean-Baptiste LAMY
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from arkanae3.rooms import Room

room = Room([
  4,  2,  2,  2,  2,  6,  6,  6,  6,
  4,  4,  0,  0,  0,  0,  7,  7,  7,
  4,  4,  0,  0,  0,  0,  7,  7,  7,
  4,  7,  7,  0,  0,  0,  0,  0,  2,
  4,  7,  7,  0,  0,  0,  0,  0,  2,
  4,  7,  7,  4,  0,  4,  4,  0,  2,
  4,  4,  4,  4,  2,  4,  4,  4,  4,
  ],
  )

room.visible_rooms = [(-1, 1), (0, 1), (1, 1), (-1, 0), (0, 0), (1, 0), (0, -1)]

#room.y_offset = -4.0
room.atmosphere = 2

def build(room_builder):
  room_builder.build_water("eau_bleu")
  room_builder.build_terrain(["grass7_1", "grass7_2", "grass7_3", "grass7_1", "grass7_2", "grass7_3", "grass7_1", "grass7_2", "grass7_3", "grass7_1", "grass7_2", "grass7_3", "grass7_4"], "grass_t1_t2", "grass_t2_t2", "sol_bis", "grass4_t1", "grass4_t2", "grass4")
  #room_builder.build_floor("pentacle3")
  #room_builder.build_wall ()
  
  import soya
  b = soya.Body(room_builder.room_world, soya.Model.get("camp1"))
  b.set_xyz(2.5, 0.0, -1.0)
  b = soya.Body(room_builder.room_world, soya.Model.get("camp2"))
  b.set_xyz(-2.2, 0.0, 1.6)
  
  
def prepare(level):
  import arkanae3.game
  #arkanae3.game.Chest().prepare(level, 1, 3.5, 1.5, arkanae3.game.Club)
  #arkanae3.game.Ork().prepare(level, 0, 3.5, 3.5)

  arkanae3.game.VoleurEchassien().prepare(level, 1, 2.5, 2.5, items = [], faction = 0, discute_script = Discussion.to_script("0"))


import arkanae3.base, arkanae3.game
class Discussion(arkanae3.base.BaseDiscussion):
  def add_options(self):
    self.add_option(discu = "marchand")
    if (self.hero.spcs.miniquete_3barbares < 10):
      self.add_option(discu = "quete")
    if (0 < self.hero.spcs.miniquete_3barbares < 10) and self.hero.has_item(arkanae3.game.Lance):
      self.add_option(discu = "lance")
    self.add_option(discu = "fin")
    
  def discu_0(self):
    self.set_text(_(u"""Héran : Bonjour, cher client !"""))
    self.add_options()
    
  def discu_marchand(self):
    u"""Vous êtes un marchand ?"""
    self.set_text(_(u"""Héran : Parfaitement. Ma caravane a été prise dans la catastrophe, mais j'ai sauvé une partie de mon stock. Je pratique le troc, et je vous propose d'échanger l'un de vos objets contre l'un des miens."""))
    self.add_option()
    
  def discu_marchand2(self):
    u"""Faites-moi voir vos objets."""
    self.set_text(_(u"""Héran : Les voici. Lequel vous intéresse ?"""))
    items = [arkanae3.game.LifePotion(), arkanae3.game.Knife(), arkanae3.game.Staff(), arkanae3.game.Robe(), arkanae3.game.Plastron()]
    if self.hero.spcs.miniquete_3barbares == 10:
      items.append(arkanae3.game.Lance())
    self.item_exchange(items)
    
  def discu_quete(self):
    u"""Avez-vous besoin de mes services ?"""
    self.set_text(_(u"""Héran : Hélas oui. Trois barbares ont profité de la catastrophe pour me voler une lance. Ils se cachent dans les ruines du port, au sud. Allez voir Knab à la buvette, il vous en apprendra un peu plus sur les barbares."""))
    self.hero.spcs.miniquete_3barbares = max(1, self.hero.spcs.miniquete_3barbares)
    self.add_options()
    
  def discu_lance(self):
    u"""J'ai retrouvé votre lance."""
    self.set_text(_(u"""Héran : Merci beaucoup ! Grâce aux lances, il nous sera plus facile de défendre le camp ! Je vais dire à Ghelled combien vous m'avez été utile !"""))
    self.hero.spcs.miniquete_3barbares = 10
    self.hero.send_message(arkanae3.base.MESSAGE_LOST + self.hero.has_item(arkanae3.game.Lance).dumpsuid())
    self.add_options()
    
  def discu_fin(self):
    u"Au revoir."
    self.set_text(u"")
