# -*- coding: utf-8 -*-
# Arkanae III
# Copyright (C) 2008-2009 Jean-Baptiste LAMY
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from arkanae3.rooms import Room

room = Room([
  6,  6,  6,  6,  6,  6,  6,  6,  6,
  7,  7,  7,  7,  7,  7,  7,  7,  7,
  7,  7,  7,  7,  7,  7,  7,  7,  7,
  2, -1, -1, -1, -1, -1, -1, -1,  2,
  2, -1, -1, -1, -1, -1, -1, -1,  2,
  2, -1, -1, -1, -1, -1, -1, -1,  2,
  4,  4,  4,  4,  4,  4,  4,  4,  4,
  ],
  )

room.visible_rooms = [(-1, 1), (0, 1), (1, 1), (-1, 0), (0, 0), (1, 0), (0, -1)]

#room.y_offset = -4.0
room.atmosphere = 2

def build(room_builder):
  room_builder.build_water("eau_bleu")
  room_builder.build_terrain(["grass7_1", "grass7_2", "grass7_3", "grass7_1", "grass7_2", "grass7_3", "grass7_1", "grass7_2", "grass7_3", "grass7_1", "grass7_2", "grass7_3", "grass7_4"], "grass_t1_t2", "grass_t2_t2", "sol_bis", "grass4_t1", "grass4_t2", "grass4")
  #room_builder.build_floor("pentacle3")
  #room_builder.build_wall ()
  
  import soya
  b = soya.Body(room_builder.room_world, soya.Model.get("camp7"))
  b.set_xyz(-2.5, 0.0, -1.2)
  b = soya.Body(room_builder.room_world, soya.Model.get("camp8"))
  b.set_xyz(-0.5, 0.0, -1.2)
  b = soya.Body(room_builder.room_world, soya.Model.get("camp9"))
  b.set_xyz( 1.5, 0.0, -1.4)
  b = soya.Body(room_builder.room_world, soya.Model.get("camp2"))
  b.set_xyz( 3.5, 0.0, -1.2)
  b.rotate_lateral(85.0)
  
  b = soya.Body(room_builder.room_world, soya.Model.get("tabouret"))
  b.set_xyz(-2.5, 0.0, -1.0)
  
  b = soya.Body(room_builder.room_world, soya.Model.get("table"))
  b.set_xyz(3.1, 0.0, -1.2)
  
def prepare(level):
  import arkanae3.game
  #arkanae3.game.Chest().prepare(level, 1, 3.5, 1.5, arkanae3.game.Club)
  #arkanae3.game.Ork().prepare(level, 0, 3.5, 3.5)
  if (not arkanae3.base.UNIVERSE_DATA.states.ech_couic) and arkanae3.base.UNIVERSE_DATA.states.ech_gueri:
    arkanae3.game.ChevalierEchassien().prepare(level, 1, 3.0, 2.5, items = [arkanae3.game.Staff(), arkanae3.game.Robe()], faction = 0, discute_script = Discussion.to_script("0"), angle = -90.0)
  arkanae3.game.BarbareFide().prepare(level, 2, 5.0, 2.5, items = [arkanae3.game.Club()], faction = 0, discute_script = Discussion2.to_script("0"), angle = 90.0)
    


import arkanae3.base
class Discussion(arkanae3.base.BaseDiscussion):
  def add_options(self):
    if (self.hero.spcs.miniquete_monstre < 10):
      self.add_option(discu = "quete")
      
    if (self.hero.spcs.miniquete_monstre == 1) and (self.hero.spcs.miniquete_monstre_nb == 3):
      self.add_option(discu = "monstre")
    self.add_option(discu = "monstre_vert")
    self.add_option(discu = "fin")
    
  def discu_0(self):
    #self.set_text(_(u"""Ech : Vous tombez bien ! J'étais en train d'expliquer aux reptiles qu'après la catastrophe, nous devons redoubler de foi envers Akéron pour implorer son pardon !"""))
    self.set_text(_(u"""Ech : Vous tombez bien ! J'étais en train d'expliquer aux reptiles que la catastrophe prouve la toute puissance d'Akéron ! Nous devons vivre dans le respect et la crainte d'Akéron."""))
    self.add_options()
    
  def discu_quete(self):
    u"""Avez-vous besoin de mes services ?"""
    self.set_text(_(u"""Ech : Oui. Je suis à présent responsable de la sécurité du camp, et un groupe de monstres s'est rassemblé sur une île à l'est. Mais je suis encore en convalescence... Pouvez-vous les éliminer ?"""))
    self.hero.spcs.miniquete_monstre = max(1, self.hero.spcs.miniquete_monstre)
    self.add_options()
    
  def discu_monstre(self):
    u"""J'ai tué le groupe de monstre à l'est"""
    self.set_text(_(u"""Ech : Très bien ! Vous avez rendu un grand service au camp !"""))
    self.hero.spcs.miniquete_monstre = 10
    self.add_options()
    
  def discu_monstre_vert(self):
    u"""Savez-vous d'où viennent tous ces monstres ?"""
    self.set_text(_(u"""Ech : Ce sont les corps de marins noyés depuis des siècles. La catastrophe les a ramené à la surface... il n'y a pas d'autre moyen de s'en débarrasser que de les combattre !"""))
    self.add_options()
    
  def discu_fin(self):
    u"Au revoir"
    self.set_text(u"")


import arkanae3.base
class Discussion2(arkanae3.base.BaseDiscussion):
  def discu_bloc_1_enter(self):
    self.set_text(u"Voici la buvette du camp. Il y a de l'animation !")
    
  def add_options(self):
    self.add_option(discu = "tete")
    if self.hero.spcs.miniquete_3barbares:
      self.add_option(discu = "barbare")
    if (self.hero.spcs.miniquete_bateau < 10):
      self.add_option(discu = "quete")
    if (self.hero.spcs.miniquete_bateau == 1) and self.hero.spcs.barque_intacte_decouverte:
      self.add_option(discu = "bateau")
    self.add_option(discu = "fin")
    
  def discu_0(self):
    if arkanae3.base.UNIVERSE_DATA.states.ech_couic or (not arkanae3.base.UNIVERSE_DATA.states.ech_gueri):
      self.set_text(_(u"""Knab : Viens boire un verre avec moi ! Moquons-nous de la colère d'Akéron en buvant joyeusement !"""))
    else:
      self.set_text(_(u"""Knab : Viens boire un verre avec nous ! Tu vas m'aider à clouer le bec de ce chevalier ! Moquons-nous de la colère d'Akéron et buvons joyeusement !"""))
    self.add_options()
    
  def discu_tete(self):
    u"Pourquoi avez-vous une tête de serpent ?"
    self.set_text(_(u"""Knab : Je suis un barbare Fide. Et nous, les Fides, nous ne vénérons pas Akéron... y'a même des Fides qui veulent le détruire ! Mais moi je reste à l'écart de ces trucs là."""))
    self.add_options()
    
  def discu_barbare(self):
    u"Que pouvez-vous m'apprendre sur les barbares ?"
    self.set_text(_(u"""Knab : Nous, barbares du sud, on est les guerriers les plus forts ! Quand on est à demi-mort, on entre dans une rage terrible. Retiens ça, petit : si tu dois affronter un barbare, garde tes forces pour la fin du combat !"""))
    self.add_option()
    
  def discu_barbare2(self):
    u"Suite"
    self.set_text(_(u"""Knab : Et s'il s'agit des barbares qui ont volé les lances d'Héran, tu devrais te munir d'une arme à grande allonge..."""))
    self.add_options()
    
  def discu_quete(self):
    u"""Avez-vous besoin de mes services ?"""
    self.set_text(_(u"""Knab : Avant la catastrophe, j'pêchais souvent dans les marais du sud. Si tu trouves un bateau encore utilisable, je pourrai ramener du poisson pour nourrir le camp..."""))
    self.hero.spcs.miniquete_bateau = max(1, self.hero.spcs.miniquete_bateau)
    self.add_options()
    
  def discu_bateau(self):
    u"""J'ai trouvé un bateau intact sur la côte."""
    self.set_text(_(u"""Knab : Super ! On va pouvoir ajouter du poisson au menu de la buvette ! Je ferai savoir à Ghelled que tu m'as aidé."""))
    self.hero.spcs.miniquete_bateau = 10
    self.add_options()
    
  def discu_fin(self):
    u"Au revoir."
    self.set_text(u"")
    
