# -*- coding: utf-8 -*-
# Arkanae III
# Copyright (C) 2008-2009 Jean-Baptiste LAMY
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from arkanae3.rooms import Room

room = Room([
  4,  4,  4,  4,  4,  4,  4,  4,  4,
  4,  4,  4,  7,  7,  7,  7,  7,  4,
  4,  0,  0,  7,  7,  7,  7,  7,  4,
  4,  0,  0,  7,  7,  7,  0,  0,  4,
  4,  0,  0,  0,  0,  0,  0,  0,  4,
  4,  0,  0,  0,  7,  7,  0,  0,  4,
  4,  4,  2,  2,  7,  7,  2,  4,  4,
  ],
  )

room.visible_rooms = [(-1, 1), (0, 1), (1, 1), (-1, 0), (0, 0), (1, 0), (0, -1)]

room.y_offset = 4.0
room.atmosphere = 2

def build(room_builder):
  room_builder.build_terrain(["grass7_1", "grass7_2", "grass7_3"], "grass_t1_t2", "grass_t2_t2", "sol_bis", "grass4_t3", "grass4_t4", "sol_bis")
  #room_builder.build_floor("pentacle3")
  #room_builder.build_wall ()
  
  import soya
  b = soya.Body(room_builder.room_world, soya.Model.get("camp6"))
  b.set_xyz(0.0, 0.0, -1.0)
  b.rotate_lateral(-90.0)
  
  b = soya.Body(room_builder.room_world, soya.Model.get("table"))
  b.set_xyz(0.5, 0.0, 2.0)
  b = soya.Body(room_builder.room_world, soya.Model.get("tabouret"))
  b.set_xyz(-0.2, 0.0, 2.0)
  b = soya.Body(room_builder.room_world, soya.Model.get("tabouret"))
  b.set_xyz(1.3, 0.0, 2.0)
  
def prepare(level):
  import arkanae3.game
  #arkanae3.game.Chest().prepare(level, 1, 3.5, 1.5, arkanae3.game.Club)
  #arkanae3.game.Ork().prepare(level, 0, 3.5, 3.5)
  arkanae3.game.InvocateurLige().prepare(level, 1, 0.8, 2.5, items = [arkanae3.game.Staff(), arkanae3.game.Robe()], faction = 0, discute_script = Discussion.to_script("0"))


import arkanae3.base
class Discussion(arkanae3.base.BaseDiscussion):
  def add_options(self):
    self.add_option(discu = "monde")
    nb = 0
    if self.hero.spcs.miniquete_nego      == 10: nb += 1
    if self.hero.spcs.miniquete_bateau    == 10: nb += 1
    if self.hero.spcs.miniquete_monstre   == 10: nb += 1
    if self.hero.spcs.miniquete_relique   == 10: nb += 1
    if self.hero.spcs.miniquete_3barbares == 10: nb += 1
    if nb >= 3:
      self.add_option(discu = "3quete")
      
    elif self.hero.spcs.miniquete_nego < 10:
      self.add_option(discu = "quete")
      
    self.add_option(discu = "fin")
   
  def discu_0(self):
    self.set_text(_(u"""Ghelled : Re-bonjour. J'essaie d'organiser le camp au mieux, mais nous manquons de tout..."""))
    self.add_options()
    
  def discu_3quete(self):
    u"""J'ai aidé trois réfugiés."""
    self.set_text(_(u"""Ghelled : Merci ! Grâce à vous, le camp est en ordre et les travaux vont pouvoir commencer pour dégager la route vers l'est. Ce n'est plus qu'une question de temps... en attendant, Sriss m'a dit qu'il avait une tâche à vous confier ; vous devriez aller le voir."""))
    self.add_options()
    arkanae3.base.UNIVERSE_DATA.states.sriss_dans_tente = 1
    
  def discu_quete(self):
    u"""Avez-vous besoin de mes services ?"""
    self.set_text(_(u"""Ghelled : En fait, oui. Pour reconstruire la route vers l'est, j'ai besoin de l'aide des Fides. Mais Sriss, leur représentant dans ce camp, ne veut pas me l'accorder. Si vous pouviez essayer de négocier avec lui..."""))
    self.hero.spcs.miniquete_nego = max(1, self.hero.spcs.miniquete_nego)
    self.add_options()
    
  def discu_monde(self):
    u"""J'aimerais en savoir un peu plus sur le monde d'Arkanae ?"""
    self.set_text(_(u"""Ghelled : Vous n'avez toujours pas retrouvé la mémoire ? Eh bien, le monde d'Arkanae est divisé en quatre régions. L'ouest a été ravagé par la catastrophe ; heureusement c'était une région sauvage et peu peuplée."""))
    self.add_option()
    
  def discu_monde2(self):
    u"""Suite"""
    self.set_text(_(u"""Ghelled : Dans les montagnes du nord vivent les échassiens. Ils ont construit une grande cathédrale à la gloire d'Akéron et du cristal sacré de l'Arkanae. A l'opposé, dans les marais du sud, vous trouverez les Fides."""))
    self.add_option()
    
  def discu_monde3(self):
    u"""Suite"""
    self.set_text(_(u"""Ghelled : Quand à mon peuple, les Liges, nous habitons plus loin à l'est. Nous sommes un peuple raffiné et cultivé, et les merveilles du palais des Liges sont réputées dans le monde entier."""))
    self.add_options()
    
  def discu_fin(self):
    u"Au revoir."
    self.set_text(u"")
