# -*- coding: utf-8 -*-
# Arkanae III
# Copyright (C) 2008-2009 Jean-Baptiste LAMY
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from arkanae3.rooms import Room

room = Room([
  6,  2,  2,  2,  2,  7,  7,  7,  7,
  7,  0,  0,  0,  0,  7,  7,  7,  7,
  7,  0,  0,  0,  0,  0,  0,  0,  2,
  2,  0,  0,  0,  0,  0,  0,  7,  7,
  2,  0,  0,  4,  0,  0,  0,  0,  2,
  2,  0,  4,  4,  4,  0,  0,  0,  2,
  4,  4,  4,  4,  4,  4,  4,  4,  6,
  ],
  )

room.visible_rooms = [(-1, 1), (0, 1), (1, 1), (-1, 0), (0, 0), (1, 0), (0, -1)]

#room.y_offset = -4.0
room.atmosphere = 2

def build(room_builder):
  room_builder.build_water("eau_bleu")
  room_builder.build_terrain(["grass7_1", "grass7_2", "grass7_3", "grass7_1", "grass7_2", "grass7_3", "grass7_1", "grass7_2", "grass7_3", "grass7_1", "grass7_2", "grass7_3", "grass7_4"], "grass_t1_t2", "grass_t2_t2", "sol_bis", "grass4_t1", "grass4_t2", "grass4")
  #room_builder.build_floor("pentacle3")
  #room_builder.build_wall ()
  
  import soya
  b = soya.Body(room_builder.room_world, soya.Model.get("camp5"))
  b.set_xyz(2.0, 0.0, -0.5)
  b.rotate_lateral(-90.0)
  
def prepare(level):
  import arkanae3.game
  #arkanae3.game.Chest().prepare(level, 1, 3.5, 1.5, arkanae3.game.Club)
  #arkanae3.game.Ork().prepare(level, 0, 3.5, 3.5)
  if arkanae3.base.UNIVERSE_DATA.states.sriss_dans_tente:
    arkanae3.game.BarbareFide().prepare(level, 2, 4.5, 2.5, items = [arkanae3.game.Axe()], discute_script = BarbareDiscussion.to_script("0"), faction = 0)
  else:
    arkanae3.game.SorcierFide().prepare(level, 2, 4.5, 2.5, items = [arkanae3.game.Knife()], discute_script = SrissDiscussion.to_script("0"), faction = 0)



import arkanae3.base, arkanae3.game

class BarbareDiscussion(arkanae3.base.BaseDiscussion):
  def add_options(self):
    self.add_option(discu = "fin")
    
  def discu_0(self):
    self.set_text(_(u"""Barbare : Bonjour. Sriss veut vous parler, il vous attend sous ses tentes, à droite. Vous pouvez y pénétrer."""))
    self.add_options()
    
  def discu_fin(self):
    u"Au revoir."
    self.set_text(u"")
    
 
class SrissDiscussion(arkanae3.base.BaseDiscussion):
  def add_options(self):
    self.add_option(discu = "tete")
    self.add_option(discu = "tente")
    if self.hero.spcs.miniquete_nego == 1:
      self.add_option(discu = "nego")
    if self.hero.has_item(arkanae3.game.SrissPaper) and self.hero.spcs.miniquete_nego_papier_sriss:
      self.add_option(discu = "papier_sriss")
      
    self.add_option(discu = "quete")
    self.add_option(discu = "fin")
    
  def discu_0(self):
    self.set_text(_(u"""Sriss : Bonjour, sss..."""))
    self.add_options()
    
  def discu_tete(self):
    u"Pourquoi avez-vous une tête de serpent ?"
    self.set_text(_(u"""Sriss : Je suis un sorcier Fide, des marais du sud."""))
    self.add_options()
    
  def discu_tente(self):
    u"Qu'y a-t-il sous vos tentes ?"
    self.set_text(_(u"""Sriss : Cela ne vous regarde pas. N'y pénétrez pas, ou bien mes gardes seront obligés de vous attaquer !"""))
    self.add_options()
    
  def discu_nego(self):
    u"Ghelled a besoin de votre aide pour dégager la route."
    self.set_text(_(u"""Sriss : Il m'en a déjà parlé, mais il n'a pas les moyens de me payer. C'est une affaire entre lui et moi, qui ne vous regarde pas."""))
    self.add_options()
    
  def discu_papier_sriss(self):
    u"J'ai trouvé ces papiers compromettants."
    self.set_text(_(u"""Sriss : Ces papiers sont à moi. Ils n'ont plus aucune valeur depuis la catastrophe. Veuillez me les rendre, monsieur le voleur !"""))
    if self.hero.spcs.miniquete_nego:
      self.add_option(discu = "papier_sriss3")
    else:
      self.add_option(discu = "papier_sriss2")
      
  def discu_papier_sriss2(self):
    u"Je vois que je n'ai pas le choix."
    self.set_text(_(u"""Sriss : En effet. Merci."""))
    self.hero.send_message(arkanae3.base.MESSAGE_LOST + self.hero.has_item(arkanae3.game.SrissPaper).dumpsuid())
    self.add_options()
    
  def discu_papier_sriss3(self):
    u"Je vous propose un marché : les papiers contre votre aide pour Ghelled."
    self.set_text(_(u"""Sriss : Je vois que je n'ai pas le choix... C'est entendu, les Fides aideront à dégager la route."""))
    self.hero.spcs.miniquete_nego = 10
    self.hero.send_message(arkanae3.base.MESSAGE_LOST + self.hero.has_item(arkanae3.game.SrissPaper).dumpsuid())
    self.add_options()
    
  def discu_quete(self):
    u"""Avez-vous besoin de mes services ?"""
    self.set_text(_(u"""Sriss : Pas pour l'instant, mais cela pourrait changer. Je vous le ferai savoir."""))
    self.add_options()
    
  def discu_fin(self):
    u"Au revoir."
    self.set_text(u"")


