# -*- coding: utf-8 -*-
# Arkanae III
# Copyright (C) 2008-2009 Jean-Baptiste LAMY
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from arkanae3.rooms import Room

room = Room([
  6,  6,  6,  6,  2,  2,  4,  4,  4,
  6,  7,  7, -1, -1, -1, -1,  4,  4,
  6,  7,  7,  0,  0,  0,  0,  0,  4,
  6,  1,  0,  0,  0,  0,  0,  0,  2,
  6,  0,  0,  0,  0,  0,  0,  0,  4,
  6,  0,  0,  0,  0,  0,  0,  0,  4,
  6,  2,  2,  2,  2,  7,  7,  7,  7,
  ],
  )

room.visible_rooms = [(-1, 1), (0, 1), (1, 1), (-1, 0), (0, 0), (1, 0), (0, -1)]

#room.y_offset = -4.0
room.atmosphere = 2

def build(room_builder):
  room_builder.build_water("eau_bleu")
  room_builder.build_terrain(["grass7_1", "grass7_2", "grass7_3", "grass7_1", "grass7_2", "grass7_3", "grass7_1", "grass7_2", "grass7_3", "grass7_1", "grass7_2", "grass7_3", "grass7_4"], "grass_t1_t2", "grass_t2_t2", "sol_bis", "grass4_t1", "grass4_t2", "grass4")
  #room_builder.build_floor("pentacle3")
  #room_builder.build_wall ()
  
  import soya
  b = soya.Body(room_builder.room_world, soya.Model.get("camp2"))
  b.set_xyz(-2.5, 0.0, -1.5)
  
def prepare(level):
  import arkanae3.game
  arkanae3.game.Fountain().prepare(level, 5.5, 3.5)
  #arkanae3.game.Chest().prepare(level, 1, 3.5, 1.5, arkanae3.game.Club)
  #arkanae3.game.Ork().prepare(level, 0, 3.5, 3.5)
  arkanae3.game.PretreEchassien().prepare(level, 2, 5.5, 1.0, items = [arkanae3.game.Robe()], discute_script = HiboulDiscussion.to_script("0"), faction = 0)


import arkanae3.base, arkanae3.game
class HiboulDiscussion(arkanae3.base.BaseDiscussion):
  def add_options(self):
    self.add_option(discu = "akeron")
    if self.hero.spcs.miniquete_relique < 10:
      self.add_option(discu = "quete")
      
    if (0 < self.hero.spcs.miniquete_relique < 9) and self.hero.has_item(arkanae3.game.RobeDeVerre):
      self.add_option(discu = "robe_verre")
    elif self.hero.spcs.miniquete_relique in [2, 3]:
      self.add_option(discu = "quete_reneg")
      
    if self.hero.spcs.miniquete_nego_papier_sriss:
      self.add_option(discu = "papier_sriss")
    self.add_option(discu = "fin")
    
  def discu_0(self):
    self.set_text(_(u"""Hiboul : Que la paix d'Akéron vous accompagne ! Je suis prêtre d'Akéron, et je soigne les blessés."""))
    self.add_options()
    
  def discu_akeron(self):
    u"""Qui est Akéron ?"""
    self.set_text(_(u"""Hiboul : Akéron est le dieu unique, un dieu de paix et de sérénité qui a déjà accompli bien des miracles."""))
    self.add_option()
    
  def discu_akeron2(self):
    u"""Akéron est-il responsable de la catastrophe ?"""
    self.set_text(_(u"""Hiboul : Bien sûr. La catastrophe est une punition qu'il nous a envoyée ; à présent nous devons implorer le pardon d'Akéron. Et vous, qu'en pensez-vous ?"""))
    self.add_option(discu = "akeron21")
    self.add_option(discu = "akeron22")
    
  def discu_akeron21(self):
    u"""Je pense que vous avez raison."""
    if (not self.hero.spcs.camp_hiboul_potion) or ((self.hero.spcs.camp_hiboul_potion < 5) and (not self.hero.has_item(arkanae3.game.LifePotion))):
      self.hero.spcs.camp_hiboul_potion = self.hero.spcs.camp_hiboul_potion + 1
      self.npc.add_item(arkanae3.game.LifePotion())
      self.npc.drop(self.npc.items[-1])
      self.set_text(_(u"""Hiboul : Bien, bien ! Prenez cette potion, comme récompense de votre foi !"""))
    else:
      self.set_text(_(u"""Hiboul : Bien, bien ! Qu'Akéron vous bénisse."""))
    self.add_options()
    
  def discu_akeron22(self):
    u"""Je n'ai pas confiance dans Akéron."""
    self.set_text(_(u"""Hiboul : Taisez-vous, jeune coq ! On pourrait vous prendre pour un de ces serpents révolutionnaires !"""))
    
  def discu_quete(self):
    u"""Avez-vous besoin de mes services ?"""
    self.set_text(_(u"""Hiboul : Oui. Pendant la catastrophe, j'ai perdu une relique : la robe de verre. Je me demande si un chevalier ne l'a pas emportée, mais j'ignore ce qu'il est devenu... Si vous pouviez retrouver cette relique, Akéron vous remercierait !"""))
    self.hero.spcs.miniquete_relique = max(1, self.hero.spcs.miniquete_relique)
    self.add_options()
    
  def discu_quete_reneg(self):
    u"""Reneg détient la robe de verre."""
    self.set_text(_(u"""Hiboul : Reneg... ce chevalier a toujours été faible psychologiquement. Essayez de le convaincre en lui rappelant que les voies d'Akéron sont impénétrables."""))
    self.hero.spcs.miniquete_relique = max(3, self.hero.spcs.miniquete_relique)
    self.add_options()
    
  def discu_robe_verre(self):
    u"""J'ai retrouvé la robe de verre."""
    self.set_text(_(u"""Hiboul : Merci ! Les pouvoirs de la robe m'aideront à soigner les blessés. Akéron vous le rendra !"""))
    self.hero.spcs.miniquete_relique = 10
    self.hero.send_message(arkanae3.base.MESSAGE_LOST + self.hero.has_item(arkanae3.game.RobeDeVerre).dumpsuid())
    self.add_options()
    
  def discu_papier_sriss(self):
    u"""Sriss préparait l'attaque de la cathédrale !"""
    self.set_text(_(u"""Hiboul : Cela ne me surprend pas de lui. Donc, il voulait passer par l'ouest ; mais ce n'est plus possible depuis la catastrophe."""))
    self.add_option()
    
  def discu_papier_sriss2(self):
    u"""Suite"""
    self.set_text(_(u"""Hiboul : D'ailleurs c'est sans doute pour arrêter les Fides qu'Akéron a déclenché la catastrophe..."""))
    self.add_options()
    
  def discu_fin(self):
    u"Au revoir."
    self.set_text(u"")


class Discussion(arkanae3.game.MonsterRainDiscussion):
  discu_bloc_1_enter = arkanae3.game.MonsterRainDiscussion.discu_abris
  
